@@ -21,8 +21,7 @@ DEFUN_DLD (unwinddemo, args, nargout, "Unwind Demo")
       if (! error_state)
         {
           unwind_protect::begin_frame ("Funwinddemo");
-          unwind_protect_ptr 
-	    (current_liboctave_warning_handler);
+          unwind_protect_ptr (current_liboctave_warning_handler);
           set_liboctave_warning_handler(err_hand);
           retval = octave_value (quotient (a, b));
           unwind_protect::run_frame ("Funwinddemo");
