@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2003, 2005, 2007 John W. Eaton
+Copyright (C) 1996-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -31,10 +31,6 @@ along with Octave; see the file COPYING.  If not, see
 #include "Array.h"
 #include "Array.cc"
 
-INSTANTIATE_ARRAY (idx_vector, OCTAVE_API);
+NO_INSTANTIATE_ARRAY_SORT (idx_vector);
 
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
+INSTANTIATE_ARRAY (idx_vector, OCTAVE_API);
