@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2001, 2002, 2004,
-              2005, 2007 John W. Eaton
+Copyright (C) 1993-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -31,9 +30,19 @@ along with Octave; see the file COPYING.  If not, see
 #include <limits>
 
 #include "Range.h"
+#include "lo-error.h"
 #include "lo-mappers.h"
 #include "lo-math.h"
 #include "lo-utils.h"
+#include "Array-util.h"
+
+Range::Range (double b, double i, octave_idx_type n)
+  : rng_base (b), rng_limit (b + (n-1) * i), rng_inc (i),
+    rng_nelem (n), cache ()
+{
+  if (! xfinite (b) || ! xfinite (i))
+    rng_nelem = -2;
+}
 
 bool
 Range::all_elements_are_ints (void) const
@@ -43,20 +52,20 @@ Range::all_elements_are_ints (void) const
   // or fewer elements only the base needs to be an integer
 
   return (! (xisnan (rng_base) || xisnan (rng_inc))
-	  && (NINTbig (rng_base) == rng_base || rng_nelem < 1)
-	  && (NINTbig (rng_inc) == rng_inc || rng_nelem <= 1));
+          && (NINTbig (rng_base) == rng_base || rng_nelem < 1)
+          && (NINTbig (rng_inc) == rng_inc || rng_nelem <= 1));
 }
 
 Matrix
 Range::matrix_value (void) const
 {
-  if (rng_nelem > 0 && cache.rows () == 0)
+  if (rng_nelem > 0 && cache.nelem () == 0)
     {
       cache.resize (1, rng_nelem);
       double b = rng_base;
       double increment = rng_inc;
       for (octave_idx_type i = 0; i < rng_nelem; i++)
-	cache(i) = b + i * increment;
+        cache(i) = b + i * increment;
 
       // On some machines (x86 with extended precision floating point
       // arithmetic, for example) it is possible that we can overshoot
@@ -65,13 +74,62 @@ Range::matrix_value (void) const
       // elements.
 
       if ((rng_inc > 0 && cache(rng_nelem-1) > rng_limit)
-	  || (rng_inc < 0 && cache(rng_nelem-1) < rng_limit))
-	cache(rng_nelem-1) = rng_limit;
+          || (rng_inc < 0 && cache(rng_nelem-1) < rng_limit))
+        cache(rng_nelem-1) = rng_limit;
     }
 
   return cache;
 }
 
+double
+Range::checkelem (octave_idx_type i) const
+{
+  if (i < 0 || i >= rng_nelem)
+    gripe_index_out_of_range (1, 1, i+1, rng_nelem);
+
+  return rng_base + rng_inc * i;
+}
+
+struct _rangeidx_helper
+{
+  double *array, base, inc;
+  _rangeidx_helper (double *a, double b, double i)
+    : array (a), base (b), inc (i) { }
+  void operator () (octave_idx_type i)
+    { *array++ = base + i * inc; }
+};
+
+Array<double>
+Range::index (const idx_vector& i) const
+{
+  Array<double> retval;
+
+  octave_idx_type n = rng_nelem;
+
+  if (i.is_colon ())
+    {
+      retval = matrix_value ().reshape (dim_vector (rng_nelem, 1));
+    }
+  else
+    {
+      if (i.extent (n) != n)
+        gripe_index_out_of_range (1, 1, i.extent (n), n); // throws
+
+      dim_vector rd = i.orig_dimensions ();
+      octave_idx_type il = i.length (n);
+
+      // taken from Array.cc.
+
+      if (n != 1 && rd.is_vector ())
+        rd = dim_vector (1, il);
+
+      retval.clear (rd);
+
+      i.loop (n, _rangeidx_helper (retval.fortran_vec (), rng_base, rng_inc));
+    }
+
+  return retval;
+}
 
 // NOTE: max and min only return useful values if nelem > 0.
 
@@ -82,16 +140,16 @@ Range::min (void) const
   if (rng_nelem > 0)
     {
       if (rng_inc > 0)
-	retval = rng_base;
+        retval = rng_base;
       else
-	{
-	  retval = rng_base + (rng_nelem - 1) * rng_inc;
+        {
+          retval = rng_base + (rng_nelem - 1) * rng_inc;
 
-	  // See the note in the matrix_value method above.
+          // See the note in the matrix_value method above.
 
-	  if (retval < rng_limit)
-	    retval = rng_limit;
-	}
+          if (retval < rng_limit)
+            retval = rng_limit;
+        }
 
     }
   return retval;
@@ -104,24 +162,24 @@ Range::max (void) const
   if (rng_nelem > 0)
     {
       if (rng_inc > 0)
-	{
-	  retval = rng_base + (rng_nelem - 1) * rng_inc;
+        {
+          retval = rng_base + (rng_nelem - 1) * rng_inc;
 
-	  // See the note in the matrix_value method above.
+          // See the note in the matrix_value method above.
 
-	  if (retval > rng_limit)
-	    retval = rng_limit;
-	}
+          if (retval > rng_limit)
+            retval = rng_limit;
+        }
       else
-	retval = rng_base;
+        retval = rng_base;
     }
   return retval;
 }
 
 void
-Range::sort (void)
+Range::sort_internal (bool ascending)
 {
-  if (rng_base > rng_limit && rng_inc < 0.0)
+  if (ascending && rng_base > rng_limit && rng_inc < 0.0)
     {
       double tmp = rng_base;
       rng_base = min ();
@@ -129,6 +187,108 @@ Range::sort (void)
       rng_inc = -rng_inc;
       clear_cache ();
     }
+  else if (! ascending && rng_base < rng_limit && rng_inc > 0.0)
+    {
+      double tmp = rng_limit;
+      rng_limit = min ();
+      rng_base = tmp;
+      rng_inc = -rng_inc;
+      clear_cache ();
+    }
+}
+
+void
+Range::sort_internal (Array<octave_idx_type>& sidx, bool ascending)
+{
+  octave_idx_type nel = nelem ();
+
+  sidx.resize (dim_vector (1, nel));
+
+  octave_idx_type *psidx = sidx.fortran_vec ();
+
+  bool reverse = false;
+
+  if (ascending && rng_base > rng_limit && rng_inc < 0.0)
+    {
+      double tmp = rng_base;
+      rng_base = min ();
+      rng_limit = tmp;
+      rng_inc = -rng_inc;
+      clear_cache ();
+      reverse = true;
+    }
+  else if (! ascending && rng_base < rng_limit && rng_inc > 0.0)
+    {
+      double tmp = rng_limit;
+      rng_limit = min ();
+      rng_base = tmp;
+      rng_inc = -rng_inc;
+      clear_cache ();
+      reverse = true;
+    }
+
+  octave_idx_type tmp = reverse ? nel - 1 : 0;
+  octave_idx_type stp = reverse ? -1 : 1;
+
+  for (octave_idx_type i = 0; i < nel; i++, tmp += stp)
+    psidx[i] = tmp;
+
+}
+
+Matrix
+Range::diag (octave_idx_type k) const
+{
+  return matrix_value ().diag (k);
+}
+
+Range
+Range::sort (octave_idx_type dim, sortmode mode) const
+{
+  Range retval = *this;
+
+  if (dim == 1)
+    {
+      if (mode == ASCENDING)
+        retval.sort_internal (true);
+      else if (mode == DESCENDING)
+        retval.sort_internal (false);
+    }
+  else if (dim != 0)
+    (*current_liboctave_error_handler) ("Range::sort: invalid dimension");
+
+  return retval;
+}
+
+Range
+Range::sort (Array<octave_idx_type>& sidx, octave_idx_type dim,
+             sortmode mode) const
+{
+  Range retval = *this;
+
+  if (dim == 1)
+    {
+      if (mode == ASCENDING)
+          retval.sort_internal (sidx, true);
+      else if (mode == DESCENDING)
+        retval.sort_internal (sidx, false);
+    }
+  else if (dim != 0)
+    (*current_liboctave_error_handler) ("Range::sort: invalid dimension");
+
+  return retval;
+}
+
+sortmode
+Range::is_sorted (sortmode mode) const
+{
+  if (rng_nelem > 1 && rng_inc < 0)
+    mode = (mode == ASCENDING) ? UNSORTED : DESCENDING;
+  else if (rng_nelem > 1 && rng_inc > 0)
+    mode = (mode == DESCENDING) ? UNSORTED : ASCENDING;
+  else
+    mode = mode ? mode : ASCENDING;
+
+  return mode;
 }
 
 std::ostream&
@@ -157,10 +317,10 @@ operator >> (std::istream& is, Range& a)
     {
       is >> a.rng_limit;
       if (is)
-	{
-	  is >> a.rng_inc;
-	  a.rng_nelem = a.nelem_internal ();
-	}
+        {
+          is >> a.rng_inc;
+          a.rng_nelem = a.nelem_internal ();
+        }
     }
 
   return is;
@@ -169,9 +329,64 @@ operator >> (std::istream& is, Range& a)
 Range
 operator - (const Range& r)
 {
-  return Range (-r.base (), -r.limit (), -r.inc ());
+  return Range (-r.base (), -r.inc (), r.nelem ());
 }
 
+Range operator + (double x, const Range& r)
+{
+  Range result (x + r.base (), r.inc (), r.nelem ());
+  if (result.rng_nelem < 0)
+    result.cache = x + r.matrix_value ();
+
+  return result;
+}
+
+Range operator + (const Range& r, double x)
+{
+  Range result (r.base () + x, r.inc (), r.nelem ());
+  if (result.rng_nelem < 0)
+    result.cache = r.matrix_value () + x;
+
+  return result;
+}
+
+Range operator - (double x, const Range& r)
+{
+  Range result (x - r.base (), -r.inc (), r.nelem ());
+  if (result.rng_nelem < 0)
+    result.cache = x - r.matrix_value ();
+
+  return result;
+}
+
+Range operator - (const Range& r, double x)
+{
+  Range result (r.base () - x, r.inc (), r.nelem ());
+  if (result.rng_nelem < 0)
+    result.cache = r.matrix_value () - x;
+
+  return result;
+}
+
+Range operator * (double x, const Range& r)
+{
+  Range result (x * r.base (), x * r.inc (), r.nelem ());
+  if (result.rng_nelem < 0)
+    result.cache = x * r.matrix_value ();
+
+  return result;
+}
+
+Range operator * (const Range& r, double x)
+{
+  Range result (r.base () * x, r.inc () * x, r.nelem ());
+  if (result.rng_nelem < 0)
+    result.cache = r.matrix_value () * x;
+
+  return result;
+}
+
+
 // C  See Knuth, Art Of Computer Programming, Vol. 1, Problem 1.2.4-5.
 // C
 // C===Tolerant FLOOR function.
@@ -222,11 +437,11 @@ tfloor (double x, double ct)
 
   double rmax = q / (2.0 - ct);
 
-  double t1 = 1.0 + floor (x);
+  double t1 = 1.0 + gnulib::floor (x);
   t1 = (ct / q) * (t1 < 0.0 ? -t1 : t1);
   t1 = rmax < t1 ? rmax : t1;
   t1 = ct > t1 ? ct : t1;
-  t1 = floor (x + t1);
+  t1 = gnulib::floor (x + t1);
 
   if (x <= 0.0 || (t1 - x) < rmax)
     return t1;
@@ -280,21 +495,15 @@ Range::nelem_internal (void) const
       // [1.8, 1.85, 1.9].
 
       if (! teq (rng_base + (n_elt - 1) * rng_inc, rng_limit))
-	{
-	  if (teq (rng_base + (n_elt - 2) * rng_inc, rng_limit))
-	    n_elt--;
-	  else if (teq (rng_base + n_elt * rng_inc, rng_limit))
-	    n_elt++;
-	}
+        {
+          if (teq (rng_base + (n_elt - 2) * rng_inc, rng_limit))
+            n_elt--;
+          else if (teq (rng_base + n_elt * rng_inc, rng_limit))
+            n_elt++;
+        }
 
       retval = (n_elt >= std::numeric_limits<octave_idx_type>::max () - 1) ? -1 : n_elt;
     }
 
   return retval;
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
