@@ -1,8 +1,7 @@
 /*
 
-Copyright (C) 1994, 1995, 1996, 1997, 2000, 2002, 2004, 2005, 2006,
-              2007 John W. Eaton
-Copyright (C) 2008, 2009 Jaroslav Hajek
+Copyright (C) 1994-2011 John W. Eaton
+Copyright (C) 2008-2009 Jaroslav Hajek
 
 This file is part of Octave.
 
@@ -43,16 +42,10 @@ public:
   AEPBALANCE (const Matrix& a, bool noperm = false,
               bool noscal = false);
 
-  AEPBALANCE (const AEPBALANCE& a) 
+  AEPBALANCE (const AEPBALANCE& a)
     : base_aepbal<Matrix, ColumnVector> (a) { }
 
   Matrix balancing_matrix (void) const;
 };
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
