@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1994, 1995, 1996, 1997, 1998, 2002, 2005, 2007
-              John W. Eaton
+Copyright (C) 1994-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -31,13 +30,14 @@ along with Octave; see the file COPYING.  If not, see
 
 // Result of a Determinant calculation.
 
-#include "dbleDET.h"
-#include "CmplxDET.h"
+#include "DET.h"
 
 // Result of a Cholesky Factorization
 
 #include "dbleCHOL.h"
 #include "CmplxCHOL.h"
+#include "floatCHOL.h"
+#include "fCmplxCHOL.h"
 
 // Result of a Hessenberg Decomposition
 
@@ -48,11 +48,15 @@ along with Octave; see the file COPYING.  If not, see
 
 #include "dbleSCHUR.h"
 #include "CmplxSCHUR.h"
+#include "floatSCHUR.h"
+#include "fCmplxSCHUR.h"
 
 // Result of a Singular Value Decomposition.
 
 #include "dbleSVD.h"
 #include "CmplxSVD.h"
+#include "floatSVD.h"
+#include "fCmplxSVD.h"
 
 // Result of an Eigenvalue computation.
 
@@ -62,6 +66,8 @@ along with Octave; see the file COPYING.  If not, see
 
 #include "dbleLU.h"
 #include "CmplxLU.h"
+#include "floatLU.h"
+#include "fCmplxLU.h"
 
 // Result of a QR decomposition.
 
@@ -72,9 +78,3 @@ along with Octave; see the file COPYING.  If not, see
 #include "CmplxQRP.h"
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
