@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007, 2008 David Bateman
-Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
+Copyright (C) 2004-2011 David Bateman
+Copyright (C) 1998-2004 Andy Adler
 
 This file is part of Octave.
 
@@ -43,7 +43,7 @@ protected:
 
   octave_sparse_params (void)
     : params (OCTAVE_SPARSE_CONTROLS_SIZE),
-      keys (OCTAVE_SPARSE_CONTROLS_SIZE) 
+      keys (OCTAVE_SPARSE_CONTROLS_SIZE)
   {
     init_keys ();
     do_defaults ();
@@ -58,10 +58,10 @@ public:
   {
     if (&a != this)
       {
-	params = a.params;
-	keys = a.keys;
+        params = a.params;
+        keys = a.keys;
       }
-    
+
     return *this;
   }
 
@@ -72,7 +72,7 @@ public:
   static void defaults (void);
 
   static void tight (void);
-  
+
   static string_vector get_keys (void);
 
   static ColumnVector get_vals (void);
@@ -98,7 +98,7 @@ private:
   void do_defaults (void);
 
   void do_tight (void);
-  
+
   string_vector do_get_keys (void) const { return keys; }
 
   ColumnVector do_get_vals (void) const { return params; }
@@ -112,14 +112,8 @@ private:
   double do_get_bandden (void);
 
   void do_print_info (std::ostream& os, const std::string& prefix) const;
-  
+
   void init_keys (void);
 };
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
