@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2006, 2007, 2008 Michael Zeising
+## Copyright (C) 2005-2011 Michael Zeising
 ##
 ## This file is part of Octave.
 ##
@@ -23,13 +23,13 @@
 ## @var{y} is a matrix with the channels represented as columns.
 ##
 ## @deftypefnx {Function File} {[@var{y}, @var{Fs}, @var{bits}] =} wavread (@var{filename})
-## Additionally return the sample rate (@var{fs}) in Hz and the number of bits 
+## Additionally return the sample rate (@var{fs}) in Hz and the number of bits
 ## per sample (@var{bits}).
 ##
 ## @deftypefnx {Function File} {[@dots{}] =} wavread (@var{filename}, @var{n})
 ## Read only the first @var{n} samples from each channel.
 ##
-## @deftypefnx {Function File} {[@dots{}] =} wavread (@var{filename},[@var{n1} @var{n2}])
+## @deftypefnx {Function File} {[@dots{}] =} wavread (@var{filename}, @var{n1} @var{n2})
 ## Read only samples @var{n1} through @var{n2} from each channel.
 ##
 ## @deftypefnx {Function File} {[@var{samples}, @var{channels}] =} wavread (@var{filename}, "size")
@@ -52,138 +52,143 @@ function [y, samples_per_sec, bits_per_sample] = wavread (filename, param)
   endif
 
   if (! ischar (filename))
-    error ("wavwrite: expecting filename to be a character string");
+    error ("wavread: FILENAME must be a character string");
   endif
 
-  # open file for binary reading
+  # Open file for binary reading.
   [fid, msg] = fopen (filename, "rb");
   if (fid < 0)
     error ("wavread: %s", msg);
   endif
-  
-  ## check for RIFF/WAVE header
-  ck_id = char (fread (fid, 4))';
-  fseek (fid, 4, SEEK_CUR);
-  wave_id = char (fread (fid, 4))';
-  if (ck_id != "RIFF" || wave_id != "WAVE")
-    fclose (fid);
-    error ("wavread: file contains no RIFF/WAVE signature");
-  endif
-  
-  ## find format chunk within the next 256 (4*64) bytes
-  i = 1;
-  while (true)
-    if (char (fread (fid, 4))' == "fmt ");
-      break;
-    endif
-    if (i++ == 64)
-      fclose (fid);
-      error ("wavread: file contains no format chunk");
-    endif
-  endwhile
 
-  ## format chunk size
-  ck_size = fread (fid, 1, "uint32", 0, BYTEORDER);         
-  
-  ## sample format code
+  ## Get file size.
+  fseek (fid, 0, "eof");
+  file_size = ftell (fid);
+  fseek (fid, 0, "bof");
+
+  ## Find RIFF chunk.
+  riff_size = find_chunk (fid, "RIFF", file_size);
+  riff_pos = ftell (fid);
+  if (riff_size == -1)
+    fclose (fid);
+    error ("wavread: file contains no RIFF chunk");
+  endif
+
+  riff_type = char (fread (fid, 4))';
+  if(! strcmp (riff_type, "WAVE"))
+    fclose (fid);
+    error ("wavread: file contains no WAVE signature");
+  endif
+  riff_pos = riff_pos + 4;
+  riff_size = riff_size - 4;
+
+  ## Find format chunk inside the RIFF chunk.
+  fseek (fid, riff_pos, "bof");
+  fmt_size = find_chunk (fid, "fmt ", riff_size);
+  fmt_pos = ftell(fid);
+  if (fmt_size == -1)
+    fclose (fid);
+    error ("wavread: file contains no format chunk");
+  endif
+
+  ## Find data chunk inside the RIFF chunk.
+  ## We don't assume that it comes after the format chunk.
+  fseek (fid, riff_pos, "bof");
+  data_size = find_chunk (fid, "data", riff_size);
+  data_pos = ftell (fid);
+  if (data_size == -1)
+    fclose (fid);
+    error ("wavread: file contains no data chunk");
+  endif
+
+  ### Read format chunk.
+  fseek (fid, fmt_pos, "bof");
+
+  ## Sample format code.
   format_tag = fread (fid, 1, "uint16", 0, BYTEORDER);
   if (format_tag != FORMAT_PCM && format_tag != FORMAT_IEEE_FLOAT)
     fclose (fid);
     error ("wavread: sample format %#x is not supported", format_tag);
   endif
 
-  ## number of interleaved channels  
+  ## Number of interleaved channels.
   channels = fread (fid, 1, "uint16", 0, BYTEORDER);
 
-  ## sample rate
+  ## Sample rate.
   samples_per_sec = fread (fid, 1, "uint32", 0, BYTEORDER);
 
-  ## bits per sample
-  fseek (fid, 6, SEEK_CUR);
+  ## Bits per sample.
+  fseek (fid, 6, "cof");
   bits_per_sample = fread (fid, 1, "uint16", 0, BYTEORDER);
 
-  ## ignore the rest of the chunk
-  fseek (fid, ck_size-16, SEEK_CUR);
-  
-  ## find data chunk
-  i = 1;
-  while (true)
-    if (char (fread (fid, 4))' == "data")
-      break;
-    endif
-    if (i++ == 64)
-      fclose (fid);
-      error ("wavread: file contains no data chunk");
-    endif
-  endwhile
+  ### Read data chunk.
+  fseek (fid, data_pos, "bof");
 
-  ## data chunk size
-  ck_size = fread (fid, 1, "uint32", 0, BYTEORDER);
-  
-  ## determine sample data type
+  ## Determine sample data type.
   if (format_tag == FORMAT_PCM)
     switch (bits_per_sample)
       case 8
         format = "uint8";
-      case 16 
+      case 16
         format = "int16";
       case 24
-	format = "uint8";
-      case 32 
+        format = "uint8";
+      case 32
         format = "int32";
       otherwise
         fclose (fid);
         error ("wavread: %d bits sample resolution is not supported with PCM",
-	       bits_per_sample);
+               bits_per_sample);
     endswitch
   else
     switch (bits_per_sample)
-      case 32 
+      case 32
         format = "float32";
-      case 64 
+      case 64
         format = "float64";
       otherwise
         fclose (fid);
         error ("wavread: %d bits sample resolution is not supported with IEEE float",
-	       bits_per_sample);
+               bits_per_sample);
     endswitch
   endif
-  
-  ## parse arguments
+
+  ## Parse arguments.
   if (nargin == 1)
-    length = 8 * ck_size / bits_per_sample;
+    length = 8 * data_size / bits_per_sample;
   else
-    if (size (param, 2) == 1)
-      ## number of samples is given
+    nparams = size (param, 2);
+    if (nparams == 1)
+      ## Number of samples is given.
       length = param * channels;
-    elseif (size (param, 2) == 2)
-      ## sample range is given
-      if (fseek (fid, (param(1)-1) * channels * (bits_per_sample/8), SEEK_CUR) < 0)
+    elseif (nparams == 2)
+      ## Sample range is given.
+      if (fseek (fid, (param(1)-1) * channels * (bits_per_sample/8), "cof") < 0)
         warning ("wavread: seeking failed");
       endif
       length = (param(2)-param(1)+1) * channels;
-    elseif (size (param, 2) == 4 && char (param) == "size")
-      ## size of the file is requested
+    elseif (nparams == 4 && char (param) == "size")
+      ## Size of the file is requested.
       fclose (fid);
-      y = [ck_size/channels/(bits_per_sample/8), channels];
-      return
+      y = [data_size/channels/(bits_per_sample/8), channels];
+      return;
     else
       fclose (fid);
       error ("wavread: invalid argument 2");
     endif
   endif
 
-  ## read samples and close file
+  ## Read samples and close file.
   if (bits_per_sample == 24)
     length *= 3;
   endif
   [yi, n] = fread (fid, length, format, 0, BYTEORDER);
   fclose (fid);
 
-  ## check data
+  ## Check data.
   if (mod (numel (yi), channels) != 0)
     error ("wavread: data in %s doesn't match the number of channels",
-	   filename);
+           filename);
   endif
 
   if (bits_per_sample == 24)
@@ -193,21 +198,44 @@ function [y, samples_per_sec, bits_per_sample] = wavread (filename, param)
   endif
 
   if (format_tag == FORMAT_PCM)
-    ## normalize samples
+    ## Normalize samples.
     switch (bits_per_sample)
       case 8
         yi = (yi - 128)/127;
       case 16
         yi /= 32767;
       case 24
-		yi /= 8388607;
+        yi /= 8388607;
       case 32
         yi /= 2147483647;
     endswitch
   endif
-  
-  ## deinterleave
+
+  ## Deinterleave.
   nr = numel (yi) / channels;
   y = reshape (yi, channels, nr)';
-  
+
+endfunction
+
+## Given a chunk_id, scan through chunks from the current file position
+## though at most size bytes.  Return the size of the found chunk, with
+## file position pointing to the start of the chunk data.  Return -1 for
+## size if chunk is not found.
+
+function chunk_size = find_chunk (fid, chunk_id, size)
+  id = "";
+  offset = 8;
+  chunk_size = 0;
+
+  while (! strcmp (id, chunk_id) && (offset < size))
+    fseek (fid, chunk_size, "cof");
+    id = char (fread (fid, 4))';
+    chunk_size = fread (fid, 1, "uint32", 0, "ieee-le");
+    ## Chunk sizes must be word-aligned (2 byte)
+    chunk_size += rem (chunk_size, 2);
+    offset = offset + 8 + chunk_size;
+  endwhile
+  if (! strcmp (id, chunk_id))
+    chunk_size = -1;
+  endif
 endfunction
