@@ -1,4 +1,4 @@
-## Copyright (C) 2003, 2005, 2007 John W. Eaton
+## Copyright (C) 2003-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -23,8 +23,18 @@
 
 ## Author: jwe
 
+## Deprecated in version 3.0
+## Matlab still has this function, so don't remove just yet.
+
 function retval = isstr (varargin)
 
+  persistent warned = false;
+  if (! warned)
+    warned = true;
+    warning ("Octave:deprecated-function",
+             "isstr is obsolete and will be removed from a future version of Octave, please use ischar instead");
+  endif
+
   retval = ischar (varargin{:});
 
 endfunction
