@@ -1,4 +1,4 @@
-## Copyright (C) 2000, 2006, 2007, 2008, 2009 Paul Kienzle
+## Copyright (C) 2000-2011 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -20,7 +20,7 @@
 ## @deftypefn  {Function File} {} cplxpair (@var{z})
 ## @deftypefnx {Function File} {} cplxpair (@var{z}, @var{tol})
 ## @deftypefnx {Function File} {} cplxpair (@var{z}, @var{tol}, @var{dim})
-## Sort the numbers @var{z} into complex conjugate pairs ordered by 
+## Sort the numbers @var{z} into complex conjugate pairs ordered by
 ## increasing real part.  Place the negative imaginary complex number
 ## first within each pair.  Place all the real numbers (those with
 ## @code{abs (imag (@var{z}) / @var{z}) < @var{tol})}) after the
@@ -36,8 +36,8 @@
 ## error if all complex numbers are not exact conjugates (to within
 ## @var{tol}).  Note that there is no defined order for pairs with identical
 ## real parts but differing imaginary parts.
-##
 ## @c Set example in small font to prevent overfull line
+##
 ## @smallexample
 ## cplxpair (exp(2i*pi*[0:4]'/5)) == exp(2i*pi*[3; 2; 4; 1; 0]/5)
 ## @end smallexample
@@ -58,14 +58,14 @@ function y = cplxpair (z, tol, dim)
 
   if (length (z) == 0)
     y = zeros (size (z));
-    return; 
+    return;
   endif
 
   if (nargin < 2 || isempty (tol))
     if (isa (z, "single"))
       tol = 100 * eps("single");
     else
-      tol = 100*eps; 
+      tol = 100*eps;
     endif
   endif
 
@@ -119,16 +119,16 @@ function y = cplxpair (z, tol, dim)
     p = n - nr(j);
     for i = 1:2:p
       if (i+1 > p)
-	error ("cplxpair could not pair all complex numbers");
+        error ("cplxpair: could not pair all complex numbers");
       endif
       [v, idx] = min (abs (z(i+1:p) - conj (z(i))));
       if (v > tol)
-	error ("cplxpair could not pair all complex numbers");
+        error ("cplxpair: could not pair all complex numbers");
       endif
       if (imag (z(i)) < 0)
-	y([i, i+1]) = z([i, idx+i]);
+        y([i, i+1]) = z([i, idx+i]);
       else
-	y([i, i+1]) = z([idx+i, i]);
+        y([i, i+1]) = z([idx+i, i]);
       endif
       z(idx+i) = z(i+1);
     endfor
@@ -146,9 +146,9 @@ endfunction
 %!assert (cplxpair(1), 1)
 %!assert (cplxpair([1+1i, 1-1i]), [1-1i, 1+1i])
 %!assert (cplxpair([1+1i, 1+1i, 1, 1-1i, 1-1i, 2]), \
-%!	  [1-1i, 1+1i, 1-1i, 1+1i, 1, 2])
+%!        [1-1i, 1+1i, 1-1i, 1+1i, 1, 2])
 %!assert (cplxpair([1+1i; 1+1i; 1; 1-1i; 1-1i; 2]), \
-%!	  [1-1i; 1+1i; 1-1i; 1+1i; 1; 2]) 
+%!        [1-1i; 1+1i; 1-1i; 1+1i; 1; 2])
 %!assert (cplxpair([0, 1, 2]), [0, 1, 2]);
 
 %!shared z
