@@ -1,4 +1,4 @@
-## Copyright (C) 2000, 2006, 2007 Kai Habel
+## Copyright (C) 2000-2011 Kai Habel
 ##
 ## This file is part of Octave.
 ##
@@ -17,26 +17,26 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {@var{z} =} cumtrapz (@var{y})
+## @deftypefn  {Function File} {@var{z} =} cumtrapz (@var{y})
 ## @deftypefnx {Function File} {@var{z} =} cumtrapz (@var{x}, @var{y})
 ## @deftypefnx {Function File} {@var{z} =} cumtrapz (@dots{}, @var{dim})
-## 
+##
 ## Cumulative numerical integration using trapezoidal method.
-## @code{cumtrapz (@var{y})} computes the cumulative integral of the 
-## @var{y} along the first non singleton dimension. If the argument 
-## @var{x} is omitted a equally spaced vector is assumed. @code{cumtrapz 
-## (@var{x}, @var{y})} evaluates the cumulative integral with respect 
+## @code{cumtrapz (@var{y})} computes the cumulative integral of the
+## @var{y} along the first non-singleton dimension.  If the argument
+## @var{x} is omitted an equally spaced vector is assumed.  @code{cumtrapz
+## (@var{x}, @var{y})} evaluates the cumulative integral with respect
 ## to @var{x}.
-##  
+##
 ## @seealso{trapz,cumsum}
 ## @end deftypefn
 
-## Author:	Kai Habel <kai.habel@gmx.de>
+## Author:      Kai Habel <kai.habel@gmx.de>
 ##
-## also: June 2000 Paul Kienzle (fixes,suggestions) 
+## also: June 2000 Paul Kienzle (fixes,suggestions)
 ## 2006-05-12 David Bateman - Modified for NDArrays
 
-function z = cumtrapz (x, y, dim)	
+function z = cumtrapz (x, y, dim)
 
   if (nargin < 1) || (nargin > 3)
     print_usage ();
@@ -50,8 +50,7 @@ function z = cumtrapz (x, y, dim)
   if (nargin == 3)
     have_x = true;
     have_dim = true;
-  endif
-  if (nargin == 2)
+  elseif (nargin == 2)
     if (! size_equal (x, y) && isscalar (y))
       dim = y;
       have_dim = true;
@@ -61,19 +60,15 @@ function z = cumtrapz (x, y, dim)
   endif
 
   if (! have_dim)
-    ## Find the first singleton dimension.
-    dim = 0;
-    while (dim < nd && sz(dim+1) == 1)
-      dim++;
-    endwhile
-    dim++;
-    if (dim > nd)
+    ## Find the first non-singleton dimension.
+    dim = find (sz > 1, 1);
+    if (isempty (dim))
       dim = 1;
     endif
   else
     dim = floor (dim);
-    if (dim < 1 || dim > nd)
-      error ("cumtrapz: invalid dimension along which to sort");
+    if (! (isscalar (dim) && 1 <= dim && dim <= nd))
+      error ("cumtrapz: invalid dimension DIM");
     endif
   endif
 
@@ -83,17 +78,17 @@ function z = cumtrapz (x, y, dim)
     idx1{i} = 1:sz(i);
   endfor
   idx2 = idx1;
-  idx1{dim} = 2 : n;	
+  idx1{dim} = 2 : n;
   idx2{dim} = 1 : (n - 1);
 
   if (! have_x)
     z = 0.5 * cumsum (x(idx1{:}) + x(idx2{:}), dim);
   else
     if (! size_equal (x, y))
-      error ("cumtrapz: x and y must have same shape");
+      error ("cumtrapz: X and Y must have the same shape");
     endif
-    z = 0.5 * cumsum ((x(idx1{:}) - x(idx2{:})) .* 
-		      (y(idx1{:}) + y(idx2{:})), dim);
+    z = 0.5 * cumsum ((x(idx1{:}) - x(idx2{:})) .*
+                      (y(idx1{:}) + y(idx2{:})), dim);
   endif
 
   sz(dim) = 1;
