@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2004, 2005,
-##               2006, 2007, 2008, 2009 John W. Eaton
+## Copyright (C) 1993-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -20,7 +19,7 @@
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} flipud (@var{x})
 ## Return a copy of @var{x} with the order of the rows reversed.  For
-## example,
+## example:
 ##
 ## @example
 ## @group
@@ -30,7 +29,7 @@
 ## @end group
 ## @end example
 ##
-## Due to the difficulty of defining which axis about which to flip the 
+## Due to the difficulty of defining which axis about which to flip the
 ## matrix @code{flipud} only work with 2-d arrays.  To flip N-d arrays
 ## use @code{flipdim} instead.
 ## @seealso{fliplr, flipdim, rot90, rotdim}
