@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2004, 2005,
-##               2006, 2007 John W. Eaton
+## Copyright (C) 1993-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,56 +17,50 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} logspace (@var{base}, @var{limit}, @var{n})
+## @deftypefn  {Function File} {} logspace (@var{base}, @var{limit})
+## @deftypefnx {Function File} {} logspace (@var{base}, @var{limit}, @var{n})
 ## Similar to @code{linspace} except that the values are logarithmically
 ## spaced from
-## @iftex
 ## @tex
 ## $10^{base}$ to $10^{limit}$.
 ## @end tex
-## @end iftex
-## @ifinfo
+## @ifnottex
 ## 10^base to 10^limit.
-## @end ifinfo
+## @end ifnottex
 ##
 ## If @var{limit} is equal to
-## @iftex
 ## @tex
 ## $\pi$,
 ## @end tex
-## @end iftex
-## @ifinfo
+## @ifnottex
 ## pi,
-## @end ifinfo
+## @end ifnottex
 ## the points are between
-## @iftex
 ## @tex
 ## $10^{base}$ and $\pi$,
 ## @end tex
-## @end iftex
-## @ifinfo
+## @ifnottex
 ## 10^base and pi,
-## @end ifinfo
+## @end ifnottex
 ## @emph{not}
-## @iftex
 ## @tex
 ## $10^{base}$ and $10^{\pi}$,
 ## @end tex
-## @end iftex
-## @ifinfo
+## @ifnottex
 ## 10^base and 10^pi,
-## @end ifinfo
-## in order to  be compatible with the corresponding @sc{Matlab}
+## @end ifnottex
+## in order to be compatible with the corresponding @sc{matlab}
 ## function.
+## If @var{n} is unspecified it defaults to 50.
 ##
-## Also for compatibility, return the second argument if fewer than two
-## values are requested.
+## Also for compatibility with @sc{matlab}, return the second argument if
+## fewer than two values are requested.
 ## @seealso{linspace}
 ## @end deftypefn
 
 ## Author: jwe
 
-function retval = logspace (x1, x2, n)
+function retval = logspace (base, limit, n)
 
   if (nargin == 2)
     npoints = 50;
@@ -81,14 +74,30 @@ function retval = logspace (x1, x2, n)
     print_usage ();
   endif
 
-  if (length (x1) == 1 && length (x2) == 1)
-    x2_tmp = x2;
-    if (x2 == pi)
-      x2_tmp = log10 (pi);
+  if (length (base) == 1 && length (limit) == 1)
+    if (limit == pi)
+      limit = log10 (pi);
     endif
-    retval = 10 .^ (linspace (x1, x2_tmp, npoints));
+    retval = 10 .^ (linspace (base, limit, npoints));
   else
     error ("logspace: arguments must be scalars");
   endif
 
 endfunction
+
+%!test
+%! x1 = logspace (1, 2);
+%! x2 = logspace (1, 2, 10);
+%! x3 = logspace (1, -2, 10);
+%! x4 = logspace (1, pi, 10);
+%! assert((size (x1) == [1, 50] && x1(1) == 10 && x1(50) == 100
+%! && size (x2) == [1, 10] && x2(1) == 10 && x2(10) == 100
+%! && size (x3) == [1, 10] && x3(1) == 10 && x3(10) == 0.01
+%! && size (x4) == [1, 10] && x4(1) == 10 && abs (x4(10) - pi) < sqrt (eps)));
+
+%!error logspace ([1, 2; 3, 4], 5, 6);
+
+%!error logspace ();
+
+%!error logspace (1, 2, 3, 4);
+
