@@ -1,4 +1,4 @@
-## Copyright (C) 2005, 2008 Paul Kienzle
+## Copyright (C) 2005-2011 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -22,8 +22,10 @@
 ## vector of lengths and the value that was repeated.
 ##
 ## @example
+## @group
 ## runlength ([2, 2, 0, 4, 4, 4, 0, 1, 1, 1, 1])
 ## @result{}  [2, 1, 3, 1, 4]
+## @end group
 ## @end example
 ## @end deftypefn
 
