@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2004, 2005,
-##               2006, 2007, 2008, 2009 John W. Eaton
+## Copyright (C) 1993-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,19 +17,19 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} rank (@var{a}, @var{tol})
-## Compute the rank of @var{a}, using the singular value decomposition.
-## The rank is taken to be the number of singular values of @var{a} that
+## @deftypefn {Function File} {} rank (@var{A}, @var{tol})
+## Compute the rank of @var{A}, using the singular value decomposition.
+## The rank is taken to be the number of singular values of @var{A} that
 ## are greater than the specified tolerance @var{tol}.  If the second
 ## argument is omitted, it is taken to be
 ##
 ## @example
-## tol = max (size (@var{a})) * sigma(1) * eps;
+## tol = max (size (@var{A})) * sigma(1) * eps;
 ## @end example
 ##
 ## @noindent
 ## where @code{eps} is machine precision and @code{sigma(1)} is the largest
-## singular value of @var{a}.
+## singular value of @var{A}.
 ## @end deftypefn
 
 ## Author: jwe
@@ -43,9 +42,9 @@ function retval = rank (A, tol)
       tolerance = 0;
     else
       if (isa (A, "single"))
-	tolerance = max (size (A)) * sigma (1) * eps ("single");
+        tolerance = max (size (A)) * sigma (1) * eps ("single");
       else
-	tolerance = max (size (A)) * sigma (1) * eps;
+        tolerance = max (size (A)) * sigma (1) * eps;
       endif
     endif
   elseif (nargin == 2)
