@@ -1,5 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 1999, 2000, 2002, 2005, 2006, 2007, 2008
-##               Kurt Hornik
+## Copyright (C) 1995-2011 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -33,9 +32,10 @@ function z = xor (x, y)
 
   if (nargin == 2)
     if (isscalar (x) || isscalar (y) || size_equal (x, y))
-      z = logical ((x | y) - (x & y));
+      ## Typecast to logicals is necessary for other numeric types.
+      z = logical (x) != logical (y);
     else
-      error ("xor: x and y must be of common size or scalars");
+      error ("xor: X and Y must be of common size or scalars");
     endif
   else
     print_usage ();
