@@ -1,4 +1,4 @@
-## Copyright (C) 2007 David BAteman
+## Copyright (C) 2007-2011 David BAteman
 ##
 ## This file is part of Octave.
 ##
@@ -17,7 +17,7 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} contour3 (@var{z})
+## @deftypefn  {Function File} {} contour3 (@var{z})
 ## @deftypefnx {Function File} {} contour3 (@var{z}, @var{vn})
 ## @deftypefnx {Function File} {} contour3 (@var{x}, @var{y}, @var{z})
 ## @deftypefnx {Function File} {} contour3 (@var{x}, @var{y}, @var{z}, @var{vn})
@@ -27,14 +27,14 @@
 ## Plot level curves (contour lines) of the matrix @var{z}, using the
 ## contour matrix @var{c} computed by @code{contourc} from the same
 ## arguments; see the latter for their interpretation.  The contours are
-## ploted at the Z level corresponding to their contour. The set of
+## plotted at the Z level corresponding to their contour.  The set of
 ## contour levels, @var{c}, is only returned if requested.  For example:
 ##
 ## @example
 ## @group
 ## contour3 (peaks (19));
 ## hold on
-## surface (peaks (19), 'FaceColor', 'none', 'EdgeColor', 'black')
+## surface (peaks (19), "facecolor", "none", "EdgeColor", "black")
 ## colormap hot
 ## @end group
 ## @end example
@@ -43,7 +43,7 @@
 ## in a similar manner to the line styles used with the @code{plot} command.
 ## Any markers defined by @var{style} are ignored.
 ##
-## The optional input and output argument @var{h} allows an axis handle to 
+## The optional input and output argument @var{h} allows an axis handle to
 ## be passed to @code{contour} and the handles to the contour objects to be
 ## returned.
 ## @seealso{contourc, patch, plot}
@@ -57,18 +57,19 @@ function [c, h] = contour3 (varargin)
   unwind_protect
     axes (xh);
     newplot ();
-    [ctmp, htmp] = __contour__ (xh, "level", varargin{:});
+    [ctmp, htmp] = __contour__ (xh, "auto", varargin{:});
   unwind_protect_cleanup
     axes (oldh);
   end_unwind_protect
 
   if (! ishold ())
-    set (xh, "view", [-37.5, 30]);
+    set (xh, "view", [-37.5, 30],
+         "xgrid", "on", "ygrid", "on", "zgrid", "on");
   endif
 
   if (nargout > 0)
     c = ctmp;
-    h = htmp
+    h = htmp;
   endif
 
 endfunction
@@ -76,6 +77,9 @@ endfunction
 %!demo
 %! contour3 (peaks (19));
 %! hold on
-%! surface (peaks (19), 'FaceColor', 'none', 'EdgeColor', 'black')
+%! surface (peaks (19), "facecolor", "none", "edgecolor", "black")
 %! colormap hot
+%! axis tight
+%! zlim auto
 %! hold off
+%! box off
