@@ -1,5 +1,4 @@
-## Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2002, 2003,
-##               2004, 2005, 2006, 2007 John W. Eaton
+## Copyright (C) 1994-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,26 +17,54 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} hist (@var{y}, @var{x}, @var{norm})
+## @deftypefn  {Function File} {} hist (@var{y})
+## @deftypefnx {Function File} {} hist (@var{y}, @var{x})
+## @deftypefnx {Function File} {} hist (@var{y}, @var{nbins})
+## @deftypefnx {Function File} {} hist (@var{y}, @var{x}, @var{norm})
+## @deftypefnx {Function File} {[@var{nn}, @var{xx}] =} hist (@dots{})
+## @deftypefnx {Function File} {[@dots{}] =} hist (@dots{}, @var{prop}, @var{val})
+##
 ## Produce histogram counts or plots.
 ##
-## With one vector input argument, plot a histogram of the values with
-## 10 bins.  The range of the histogram bins is determined by the range
-## of the data.
+## With one vector input argument, @var{y}, plot a histogram of the values
+## with 10 bins.  The range of the histogram bins is determined by the
+## range of the data.  With one matrix input argument, @var{y}, plot a
+## histogram where each bin contains a bar per input column.
 ##
-## Given a second scalar argument, use that as the number of bins.
+## Given a second vector argument, @var{x}, use that as the centers of
+## the bins, with the width of the bins determined from the adjacent
+## values in the vector.
 ##
-## Given a second vector argument, use that as the centers of the bins,
-## with the width of the bins determined from the adjacent values in
-## the vector.
+## If scalar, the second argument, @var{nbins}, defines the number of bins.
 ##
-## If third argument is provided, the histogram is normalised such that
+## If a third argument is provided, the histogram is normalized such that
 ## the sum of the bars is equal to @var{norm}.
 ##
 ## Extreme values are lumped in the first and last bins.
 ##
 ## With two output arguments, produce the values @var{nn} and @var{xx} such
 ## that @code{bar (@var{xx}, @var{nn})} will plot the histogram.
+##
+## The histogram's appearance may be modified by specifying property/value
+## pairs, @var{prop} and @var{val} pairs.  For example the face and edge
+## color may be modified.
+##
+## @example
+## @group
+## hist (randn (1, 100), 25, "facecolor", "r", "edgecolor", "b")
+## @end group
+## @end example
+##
+## @noindent
+## The histograms colors also depend upon the colormap.
+##
+## @example
+## @group
+## hist (rand (10, 3))
+## colormap (summer ())
+## @end group
+## @end example
+##
 ## @seealso{bar}
 ## @end deftypefn
 
@@ -59,7 +86,7 @@ function [nn, xx] = hist (y, varargin)
     max_val = max (y(:));
     min_val = min (y(:));
   else
-    error ("hist: first argument must be a vector");
+    error ("hist: first argument must be real valued");
   endif
 
   iarg = 1;
@@ -79,7 +106,7 @@ function [nn, xx] = hist (y, varargin)
       x = x * (max_val - min_val) + ones (size (x)) * min_val;
     elseif (isreal (x))
       if (isvector (x))
-	x = x(:);
+        x = x(:);
       endif
       tmp = sort (x);
       if (any (tmp != x))
@@ -97,9 +124,10 @@ function [nn, xx] = hist (y, varargin)
 
   cutoff = (x(1:end-1,:) + x(2:end,:)) / 2;
   n = rows (x);
+  y_nc = columns (y);
   if (n < 30 && columns (x) == 1)
     ## The following algorithm works fastest for n less than about 30.
-    chist = zeros (n+1, columns (y));
+    chist = zeros (n+1, y_nc);
     for i = 1:n-1
       chist(i+1,:) = sum (y <= cutoff(i));
     endfor
@@ -108,13 +136,12 @@ function [nn, xx] = hist (y, varargin)
     ## The following algorithm works fastest for n greater than about 30.
     ## Put cutoff elements between boundaries, integrate over all
     ## elements, keep totals at boundaries.
-    [s, idx] = sort ([y; cutoff]);
+    [s, idx] = sort ([y; repmat(cutoff, 1, y_nc)]);
     len = rows (y);
     chist = cumsum (idx <= len);
-    t1 = zeros (1, columns (y));
-    t2 = reshape (chist(idx > len), size (cutoff));
-    t3 = chist(end,:) - sum (isnan (y));
-    chist = [t1; t2; t3];
+    chist = [(zeros (1, y_nc));
+             (reshape (chist(idx > len), rows (cutoff), y_nc));
+             (chist(end,:) - sum (isnan (y)))];
   endif
 
   freq = diff (chist);
@@ -160,9 +187,11 @@ endfunction
 %!assert(hist(1,1),1);
 %!test
 %!  for n = [10, 30, 100, 1000]
-%!    assert( sum(hist([1:n], n)), n );
-%!    assert( sum(hist([1:n], [2:n-1])), n);
-%!    assert( sum(hist([1:n], [1:n])), n );
-%!    assert( sum(hist([1:n], 29)), n);
-%!    assert( sum(hist([1:n], 30)), n);
+%!    assert(sum(hist([1:n], n)), n);
+%!    assert(sum(hist([1:n], [2:n-1])), n);
+%!    assert(sum(hist([1:n], [1:n])), n);
+%!    assert(sum(hist([1:n], 29)), n);
+%!    assert(sum(hist([1:n], 30)), n);
 %!  endfor
+%!test
+%!  assert (size (hist(randn(750,240), 200)), [200,240]);
