@@ -1,4 +1,4 @@
-## Copyright (C) 2007 Paul Kienzle
+## Copyright (C) 2007-2011 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -17,19 +17,17 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} peaks ()
+## @deftypefn  {Function File} {} peaks ()
 ## @deftypefnx {Function File} {} peaks (@var{n})
 ## @deftypefnx {Function File} {} peaks (@var{x}, @var{y})
 ## @deftypefnx {Function File} {@var{z} =} peaks (@dots{})
 ## @deftypefnx {Function File} {[@var{x}, @var{y}, @var{z}] =} peaks (@dots{})
-## Generate a function with lots of local maxima and minima. The function
+## Generate a function with lots of local maxima and minima.  The function
 ## has the form
 ##
-## @iftex
 ## @tex
 ## $f(x,y) = 3 (1 - x) ^ 2 e ^ {\left(-x^2 - (y+1)^2\right)} - 10 \left({x \over 5} - x^3 - y^5)\right) - {1 \over 3} e^{\left(-(x+1)^2 - y^2\right)}$
 ## @end tex
-## @end iftex
 ## @ifnottex
 ## @verbatim
 ## f(x,y) = 3*(1-x)^2*exp(-x^2 - (y+1)^2) ...
@@ -38,13 +36,13 @@
 ## @end verbatim
 ## @end ifnottex
 ##
-## Called without a return argument, @code{peaks} plots the surface of the 
-## above function using @code{mesh}. If @var{n} is a scalar, the @code{peaks}
+## Called without a return argument, @code{peaks} plots the surface of the
+## above function using @code{mesh}.  If @var{n} is a scalar, the @code{peaks}
 ## returns the values of the above function on a @var{n}-by-@var{n} mesh over
-## the range @code{[-3,3]}. The default value for @var{n} is 49.
+## the range @code{[-3,3]}.  The default value for @var{n} is 49.
 ##
 ## If @var{n} is a vector, then it represents the @var{x} and @var{y} values
-## of the grid on which to calculate the above function. The @var{x} and 
+## of the grid on which to calculate the above function.  The @var{x} and
 ## @var{y} values can be specified separately.
 ## @seealso{surf, mesh, meshgrid}
 ## @end deftypefn
