@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2002, 2004,
-##               2005, 2006, 2007 John W. Eaton
+## Copyright (C) 1993-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,13 +17,16 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} plot (@var{y})
+## @deftypefn  {Function File} {} plot (@var{y})
 ## @deftypefnx {Function File} {} plot (@var{x}, @var{y})
 ## @deftypefnx {Function File} {} plot (@var{x}, @var{y}, @var{property}, @var{value}, @dots{})
 ## @deftypefnx {Function File} {} plot (@var{x}, @var{y}, @var{fmt})
 ## @deftypefnx {Function File} {} plot (@var{h}, @dots{})
-## Produces two-dimensional plots.  Many different combinations of
-## arguments are possible.  The simplest form is
+## @deftypefnx {Function File} {@var{h} =} plot (@dots{})
+## Produce two-dimensional plots.
+##
+## Many different combinations of arguments are possible.  The simplest
+## form is
 ##
 ## @example
 ## plot (@var{y})
@@ -32,7 +34,7 @@
 ##
 ## @noindent
 ## where the argument is taken as the set of @var{y} coordinates and the
-## @var{x} coordinates are taken to be the indices of the elements,
+## @var{x} coordinates are taken to be the indices of the elements
 ## starting with 1.
 ##
 ## To save a plot, in one of several image formats such as PostScript
@@ -106,9 +108,6 @@
 ## @item .
 ## Set dots plot style.
 ##
-## @item L
-## Set steps plot style.
-##
 ## @item @var{n}
 ## Interpreted as the plot color if @var{n} is an integer in the range 1 to
 ## 6.
@@ -133,11 +132,14 @@
 ## @itemx ^
 ## Used in combination with the points or linespoints styles, set the point
 ## style.
+##
+## @item @@
+## Select the next unused point style.
 ## @end table
 ##
 ## The @var{fmt} argument may also be used to assign key titles.
 ## To do so, include the desired title between semi-colons after the
-## formatting sequence described above, e.g. "+3;Key Title;"
+## formatting sequence described above, e.g., "+3;Key Title;"
 ## Note that the last semi-colon is required and will generate an error if
 ## it is left out.
 ##
@@ -159,15 +161,18 @@
 ## with points displayed as @samp{*} with a marker size of 3.
 ##
 ## @example
+## @group
 ## t = 0:0.1:6.3;
 ## plot (t, cos(t), "-;cos(t);", t, sin(t), "+3;sin(t);");
+## @end group
 ## @end example
 ##
 ## This will plot the cosine and sine functions and label them accordingly
 ## in the key.
 ##
-## If the first argument is an axis handle, then plot into these axes, 
-## rather than the current axis handle returned by @code{gca}. 
+## If the first argument is an axis handle, then plot into these axes,
+## rather than the current axis handle returned by @code{gca}.
+##
 ## @seealso{semilogx, semilogy, loglog, polar, mesh, contour, bar,
 ## stairs, errorbar, xlabel, ylabel, title, print}
 ## @end deftypefn
@@ -176,7 +181,11 @@
 
 function retval = plot (varargin)
 
-  [h, varargin] = __plt_get_axis_arg__ ("plot", varargin{:});
+  [h, varargin, nargs] = __plt_get_axis_arg__ ("plot", varargin{:});
+
+  if (nargs < 1)
+    print_usage();
+  endif
 
   oldh = gca ();
   unwind_protect
