@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 2000, 2005, 2006, 2007
-##               John W. Eaton
+## Copyright (C) 1993-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -27,7 +26,7 @@
 ## @example
 ## z = sin (sqrt (x^2 + y^2)) / (sqrt (x^2 + y^2))
 ## @end example
-## @seealso{mesh, meshgrid}
+## @seealso{surf, meshgrid, mesh}
 ## @end deftypefn
 
 ## Author: jwe
@@ -45,12 +44,12 @@ function [x, y, z] = sombrero (n)
       r = sqrt (xx .^ 2 + yy .^ 2) + eps;
       tz = sin (r) ./ r;
       if (nargout == 0)
-        mesh (tx, ty, tz);
-	box ("off");
+        surf (tx, ty, tz);
+        box ("off");
       else
-	x = tx;
-	y = ty;
-	z = tz;
+        x = tx;
+        y = ty;
+        z = tz;
       endif
     else
       error ("sombrero: number of grid lines must be greater than 1");
@@ -60,3 +59,6 @@ function [x, y, z] = sombrero (n)
   endif
 
 endfunction
+
+%!demo
+%! sombrero ();
