@@ -1,5 +1,4 @@
-## Copyright (C) 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2003, 2004,
-##               2005, 2006, 2007 John W. Eaton
+## Copyright (C) 1993-2011 John W. Eaton
 ##
 ## This file is part of Octave.
 ##
@@ -18,21 +17,20 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} title (@var{title})
+## @deftypefn  {Function File} {} title (@var{string})
+## @deftypefnx {Function File} {} title (@var{string}, @var{p1}, @var{v1}, @dots{})
 ## Create a title object and return a handle to it.
 ## @end deftypefn
 
-## PKG_ADD: mark_as_command title
-
 ## Author: jwe
 
-function h = title (varargin)
+function h = title (string, varargin)
 
   if (rem (nargin, 2) == 1)
     if (nargout > 0)
-      h = __axis_label__ ("title", varargin{:});
+      h = __axis_label__ ("title", string, varargin{:});
     else
-      __axis_label__ ("title", varargin{:});
+      __axis_label__ ("title", string, varargin{:});
     endif
   else
     print_usage ();
