@@ -1,4 +1,4 @@
-## Copyright (C) 2004, 2005, 2007 Paul Kienzle
+## Copyright (C) 2004-2011 Paul Kienzle
 ##
 ## This file is part of Octave.
 ##
@@ -22,12 +22,18 @@
 ## @end deftypefn
 
 function t = nonzeros (s)
-  if (issparse (s))
-    [i, j, t] = spfind (s);
-  else
-    [i, j, t] = find (s);
+
+  if (nargin != 1)
+    print_usage ();
   endif
+
+  [i, j, t] = find (s);
+
+  t = t(:);
+
 endfunction
 
 %!assert(nonzeros([1,2;3,0]),[1;3;2])
+%!assert(nonzeros([1,2,3,0]),[1;2;3])
 %!assert(nonzeros(sparse([1,2;3,0])),[1;3;2])
+%!assert(nonzeros(sparse([1,2,3,0])),[1;2;3])
