@@ -1,5 +1,4 @@
-## Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-##               2007 Andy Adler
+## Copyright (C) 1998-2011 Andy Adler
 ##
 ## This file is part of Octave.
 ##
@@ -18,31 +17,31 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} spy (@var{x})
+## @deftypefn  {Function File} {} spy (@var{x})
 ## @deftypefnx {Function File} {} spy (@dots{}, @var{markersize})
-## @deftypefnx {Function File} {} spy (@dots{}, @var{LineSpec})
-## Plot the sparsity pattern of the sparse matrix @var{x}. If the argument
+## @deftypefnx {Function File} {} spy (@dots{}, @var{line_spec})
+## Plot the sparsity pattern of the sparse matrix @var{x}.  If the argument
 ## @var{markersize} is given as an scalar value, it is used to determine the
-## point size in the plot. If the string @var{LineSpec} is given it is
+## point size in the plot.  If the string @var{line_spec} is given it is
 ## passed to @code{plot} and determines the appearance of the plot.
 ## @seealso{plot}
 ## @end deftypefn
 
-function spy (S, varargin) 
+function spy (x, varargin)
 
   if (nargin < 1)
     print_usage ();
   endif
 
   markersize = NaN;
-  if (numel (i) < 1000)
-    LineSpec = "*";
+  if (numel (x) < 1000)
+    line_spec = "*";
   else
-    LineSpec = ".";
+    line_spec = ".";
   endif
-  for i = 1:length(varargin)
-    if (ischar(varargin{i}))
-      LineSpec = varargin{i};
+  for i = 1:length (varargin)
+    if (ischar (varargin{i}))
+      line_spec = varargin{i};
     elseif (isscalar (varargin{i}))
       markersize = varargin{i};
     else
@@ -50,17 +49,13 @@ function spy (S, varargin)
     endif
   endfor
 
-  if (issparse (S))
-    [i, j, s, m, n] = spfind (S);
-  else
-    [i, j, s] = find (S);
-    [m, n] = size (S);
-  endif
+  [i, j, s] = find (x);
+  [m, n] = size (x);
 
   if (isnan (markersize))
-    plot (j, i, LineSpec);
+    plot (j, i, line_spec);
   else
-    plot (j, i, LineSpec, "MarkerSize", markersize);
+    plot (j, i, line_spec, "markersize", markersize);
   endif
 
   axis ([0, n+1, 0, m+1], "ij");
