@@ -1,4 +1,4 @@
-## Copyright (C) 1999, 2006, 2007, 2009 Peter Ekberg
+## Copyright (C) 1999-2011 Peter Ekberg
 ##
 ## This file is part of Octave.
 ##
@@ -18,12 +18,11 @@
 
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {} rosser ()
-##
-## Returns the Rosser matrix.  This is a difficult test case used to test
+## Return the Rosser matrix.  This is a difficult test case used to evaluate
 ## eigenvalue algorithms.
 ##
 ## @seealso{hankel, vander, sylvester_matrix, hilb, invhilb, toeplitz
-##           hadamard, wilkinson, compan, pascal}
+##          hadamard, wilkinson, compan, pascal}
 ## @end deftypefn
 
 ## Author: Peter Ekberg
@@ -44,3 +43,6 @@ function retval = rosser ()
             -49,    -8,     8,    59,   208,   208,    99,  -911;
              29,   -44,    52,   -23,   208,   208,  -911,    99];
 endfunction
+
+%!assert (size(rosser()), [8,8])
+%!error (rosser(1))
