@@ -1,4 +1,4 @@
-## Copyright (C) 1995, 1996, 1997, 2005, 2006, 2007 Kurt Hornik
+## Copyright (C) 1995-2011 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -17,11 +17,11 @@
 ## <http://www.gnu.org/licenses/>.
 
 ## -*- texinfo -*-
-## @deftypefn {Function File} {} exprnd (@var{lambda}, @var{r}, @var{c})
+## @deftypefn  {Function File} {} exprnd (@var{lambda}, @var{r}, @var{c})
 ## @deftypefnx {Function File} {} exprnd (@var{lambda}, @var{sz})
 ## Return an @var{r} by @var{c} matrix of random samples from the
-## exponential distribution with parameter @var{lambda}, which must be a
-## scalar or of size @var{r} by @var{c}. Or if @var{sz} is a vector, 
+## exponential distribution with mean @var{lambda}, which must be a
+## scalar or of size @var{r} by @var{c}.  Or if @var{sz} is a vector,
 ## create a matrix of size @var{sz}.
 ##
 ## If @var{r} and @var{c} are omitted, the size of the result matrix is
@@ -31,20 +31,20 @@
 ## Author: KH <Kurt.Hornik@wu-wien.ac.at>
 ## Description: Random deviates from the exponential distribution
 
-function rnd = exprnd (l, r, c)
+function rnd = exprnd (lambda, r, c)
 
   if (nargin == 3)
     if (! (isscalar (r) && (r > 0) && (r == round (r))))
-      error ("exprnd: r must be a positive integer");
+      error ("exprnd: R must be a positive integer");
     endif
     if (! (isscalar (c) && (c > 0) && (c == round (c))))
-      error ("exprnd: c must be a positive integer");
+      error ("exprnd: C must be a positive integer");
     endif
     sz = [r, c];
 
-    if (any (size (l) != 1) && 
-	(length (size (l)) != length (sz) || any (size (l) != sz)))
-      error ("exprnd: lambda must be scalar or of size [r, c]");
+    if (any (size (lambda) != 1)
+        && (length (size (lambda)) != length (sz) || any (size (lambda) != sz)))
+      error ("exprnd: LAMBDA must be scalar or of size [R, C]");
     endif
   elseif (nargin == 2)
     if (isscalar (r) && (r > 0))
@@ -52,35 +52,35 @@ function rnd = exprnd (l, r, c)
     elseif (isvector(r) && all (r > 0))
       sz = r(:)';
     else
-      error ("exprnd: r must be a positive integer or vector");
+      error ("exprnd: R must be a positive integer or vector");
     endif
 
-    if (any (size (l) != 1) && 
-	((length (size (l)) != length (sz)) || any (size (l) != sz)))
-      error ("exprnd: lambda must be scalar or of size sz");
+    if (any (size (lambda) != 1)
+        && ((length (size (lambda)) != length (sz)) || any (size (lambda) != sz)))
+      error ("exprnd: LAMBDA must be scalar or of size SZ");
     endif
   elseif (nargin == 1)
-    sz = size (l);
+    sz = size (lambda);
   else
     print_usage ();
   endif
 
 
-  if (isscalar (l))
-    if ((l > 0) && (l < Inf))
-      rnd = rande(sz) * l;
+  if (isscalar (lambda))
+    if ((lambda > 0) && (lambda < Inf))
+      rnd = rande(sz) * lambda;
     else
-      rnd = NaN * ones (sz);
+      rnd = NaN (sz);
     endif
   else
     rnd = zeros (sz);
-    k = find (!(l > 0) | !(l < Inf));
+    k = find (!(lambda > 0) | !(lambda < Inf));
     if (any (k))
       rnd(k) = NaN;
     endif
-    k = find ((l > 0) & (l < Inf));
+    k = find ((lambda > 0) & (lambda < Inf));
     if (any (k))
-      rnd(k) = rande(size(k)) .* l(k);
+      rnd(k) = rande(size(k)) .* lambda(k);
     endif
   endif
 
