@@ -1,4 +1,4 @@
-## Copyright (C) 1996, 1999, 2000, 2004, 2005, 2006, 2007 Kurt Hornik
+## Copyright (C) 1996-2011 Kurt Hornik
 ##
 ## This file is part of Octave.
 ##
@@ -25,15 +25,17 @@
 ## the string.  If @var{len} is omitted, the substring extends to the end
 ## of S.
 ##
-## For example,
+## For example:
 ##
 ## @example
+## @group
 ## substr ("This is a test string", 6, 9)
 ##      @result{} "is a test"
+## @end group
 ## @end example
 ##
-## This function is patterned after AWK.  You can get the same result by
-## @code{@var{s} (@var{offset} : (@var{offset} + @var{len} - 1))}.
+## This function is patterned after AWK@.  You can get the same result by
+## @code{@var{s}(@var{offset} : (@var{offset} + @var{len} - 1))}.
 ## @end deftypefn
 
 ## Author: Kurt Hornik <Kurt.Hornik@wu-wien.ac.at>
@@ -69,3 +71,10 @@ function t = substr (s, offset, len)
   endif
 
 endfunction
+
+%!assert(strcmp (substr ("This is a test string", 6, 9), "is a test"));
+
+%!error substr ();
+
+%!error substr ("foo", 2, 3, 4);
+
