@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007, 2009 David Bateman
-Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
+Copyright (C) 2004-2011 David Bateman
+Copyright (C) 1998-2004 Andy Adler
 
 This file is part of Octave.
 
@@ -53,7 +53,7 @@ DEFBINOP (div, complex, sparse_complex_matrix)
       Complex d = v2.complex_value ();
 
       if (d == 0.0)
-	gripe_divide_by_zero ();
+        gripe_divide_by_zero ();
 
       return octave_value (SparseComplexMatrix (1, 1, v1.complex_value () / d));
     }
@@ -70,8 +70,8 @@ DEFBINOP (div, complex, sparse_complex_matrix)
 
 DEFBINOP (pow, complex, sparse_complex_matrix)
 {
-  CAST_BINOP_ARGS (const octave_complex&, 
-		   const octave_sparse_complex_matrix&);
+  CAST_BINOP_ARGS (const octave_complex&,
+                   const octave_sparse_complex_matrix&);
   return xpow (v1.complex_value (), v2.complex_matrix_value ());
 }
 
@@ -121,15 +121,15 @@ DEFCATOP (cs_scm, complex, sparse_complex_matrix)
 {
   CAST_BINOP_ARGS (octave_complex&, const octave_sparse_complex_matrix&);
   SparseComplexMatrix tmp (1, 1, v1.complex_value ());
-  return octave_value (tmp. concat (v2.sparse_complex_matrix_value (), 
-				    ra_idx));
+  return octave_value (tmp. concat (v2.sparse_complex_matrix_value (),
+                                    ra_idx));
 }
 
 DEFCONV (sparse_complex_matrix_conv, complex, sparse_complex_matrix)
 {
   CAST_CONV_ARG (const octave_complex&);
 
-  return new octave_sparse_complex_matrix 
+  return new octave_sparse_complex_matrix
     (SparseComplexMatrix (v.complex_matrix_value ()));
 }
 
@@ -141,38 +141,32 @@ install_cs_scm_ops (void)
   INSTALL_BINOP (op_mul, octave_complex, octave_sparse_complex_matrix, mul);
   INSTALL_BINOP (op_div, octave_complex, octave_sparse_complex_matrix, div);
   INSTALL_BINOP (op_pow, octave_complex, octave_sparse_complex_matrix, pow);
-  INSTALL_BINOP (op_ldiv, octave_complex, octave_sparse_complex_matrix, 
-		 ldiv);
+  INSTALL_BINOP (op_ldiv, octave_complex, octave_sparse_complex_matrix,
+                 ldiv);
   INSTALL_BINOP (op_lt, octave_complex, octave_sparse_complex_matrix, lt);
   INSTALL_BINOP (op_le, octave_complex, octave_sparse_complex_matrix, le);
   INSTALL_BINOP (op_eq, octave_complex, octave_sparse_complex_matrix, eq);
   INSTALL_BINOP (op_ge, octave_complex, octave_sparse_complex_matrix, ge);
   INSTALL_BINOP (op_gt, octave_complex, octave_sparse_complex_matrix, gt);
   INSTALL_BINOP (op_ne, octave_complex, octave_sparse_complex_matrix, ne);
-  INSTALL_BINOP (op_el_mul, octave_complex, octave_sparse_complex_matrix, 
-		 el_mul);
-  INSTALL_BINOP (op_el_div, octave_complex, octave_sparse_complex_matrix, 
-		 el_div);
-  INSTALL_BINOP (op_el_pow, octave_complex, octave_sparse_complex_matrix, 
-		 el_pow);
-  INSTALL_BINOP (op_el_ldiv, octave_complex, octave_sparse_complex_matrix, 
-		 el_ldiv);
-  INSTALL_BINOP (op_el_and, octave_complex, octave_sparse_complex_matrix, 
-		 el_and);
-  INSTALL_BINOP (op_el_or, octave_complex, octave_sparse_complex_matrix, 
-		 el_or);
+  INSTALL_BINOP (op_el_mul, octave_complex, octave_sparse_complex_matrix,
+                 el_mul);
+  INSTALL_BINOP (op_el_div, octave_complex, octave_sparse_complex_matrix,
+                 el_div);
+  INSTALL_BINOP (op_el_pow, octave_complex, octave_sparse_complex_matrix,
+                 el_pow);
+  INSTALL_BINOP (op_el_ldiv, octave_complex, octave_sparse_complex_matrix,
+                 el_ldiv);
+  INSTALL_BINOP (op_el_and, octave_complex, octave_sparse_complex_matrix,
+                 el_and);
+  INSTALL_BINOP (op_el_or, octave_complex, octave_sparse_complex_matrix,
+                 el_or);
 
   INSTALL_CATOP (octave_complex, octave_sparse_complex_matrix, cs_scm);
 
-  INSTALL_ASSIGNCONV (octave_complex, octave_sparse_complex_matrix, 
-		      octave_complex_matrix);
+  INSTALL_ASSIGNCONV (octave_complex, octave_sparse_complex_matrix,
+                      octave_complex_matrix);
 
-  INSTALL_WIDENOP (octave_complex, octave_sparse_complex_matrix, 
-		   sparse_complex_matrix_conv);
+  INSTALL_WIDENOP (octave_complex, octave_sparse_complex_matrix,
+                   sparse_complex_matrix_conv);
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
