@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
-              2007, 2008 John W. Eaton
+Copyright (C) 1996-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -77,12 +76,12 @@ DEFBINOP (ldiv, matrix, complex)
   return ret;
 }
 
-DEFNDBINOP_FN (lt, matrix, complex, array, complex, mx_el_lt)
-DEFNDBINOP_FN (le, matrix, complex, array, complex, mx_el_le)
-DEFNDBINOP_FN (eq, matrix, complex, array, complex, mx_el_eq)
-DEFNDBINOP_FN (ge, matrix, complex, array, complex, mx_el_ge)
-DEFNDBINOP_FN (gt, matrix, complex, array, complex, mx_el_gt)
-DEFNDBINOP_FN (ne, matrix, complex, array, complex, mx_el_ne)
+DEFNDCMPLXCMPOP_FN (lt, matrix, complex, array, complex, mx_el_lt)
+DEFNDCMPLXCMPOP_FN (le, matrix, complex, array, complex, mx_el_le)
+DEFNDCMPLXCMPOP_FN (eq, matrix, complex, array, complex, mx_el_eq)
+DEFNDCMPLXCMPOP_FN (ge, matrix, complex, array, complex, mx_el_ge)
+DEFNDCMPLXCMPOP_FN (gt, matrix, complex, array, complex, mx_el_gt)
+DEFNDCMPLXCMPOP_FN (ne, matrix, complex, array, complex, mx_el_ne)
 
 DEFNDBINOP_OP (el_mul, matrix, complex, array, complex, *)
 
@@ -139,9 +138,3 @@ install_m_cs_ops (void)
   INSTALL_ASSIGNCONV (octave_matrix, octave_complex, octave_complex_matrix);
   INSTALL_ASSIGNCONV (octave_float_matrix, octave_complex, octave_float_complex_matrix);
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
