@@ -1,7 +1,7 @@
 /*
 
-Copyright (C) 2004, 2005, 2006, 2007 David Bateman
-Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Andy Adler
+Copyright (C) 2004-2011 David Bateman
+Copyright (C) 1998-2004 Andy Adler
 
 This file is part of Octave.
 
@@ -46,29 +46,25 @@ DEFBINOP_OP (mul, sparse_complex_matrix, complex, *)
 
 DEFBINOP (div, sparse_complex_matrix, complex)
 {
-  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&, 
-		   const octave_complex&);
+  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&,
+                   const octave_complex&);
 
   Complex d = v2.complex_value ();
   octave_value retval;
 
   if (d == 0.0)
-    {
-      gripe_divide_by_zero ();
+    gripe_divide_by_zero ();
 
-      retval = (v1.complex_matrix_value ()  / d);
-    }
-  else
-    retval = octave_value (v1.sparse_complex_matrix_value () / d);
+  retval = octave_value (v1.sparse_complex_matrix_value () / d);
 
   return retval;
 }
 
 DEFBINOP (pow, sparse_complex_matrix, complex)
 {
-  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&, 
-		   const octave_complex&);
-  return xpow (v1.complex_matrix_value (), v2.scalar_value ());
+  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&,
+                   const octave_complex&);
+  return xpow (v1.complex_matrix_value (), v2.complex_value ());
 }
 
 DEFBINOP (ldiv, sparse_complex_matrix, complex)
@@ -80,9 +76,9 @@ DEFBINOP (ldiv, sparse_complex_matrix, complex)
       Complex d = v1.complex_value ();
 
       if (d == 0.0)
-	gripe_divide_by_zero ();
+        gripe_divide_by_zero ();
 
-      return octave_value (v2.complex_value () / d);
+      return octave_value (SparseComplexMatrix (1, 1, v2.complex_value () / d));
     }
   else
     {
@@ -106,21 +102,17 @@ DEFBINOP_OP (el_mul, sparse_complex_matrix, complex, *)
 
 DEFBINOP (el_div, sparse_complex_matrix, complex)
 {
-  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&, 
-		   const octave_complex&);
+  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&,
+                   const octave_complex&);
 
   octave_value retval;
 
   Complex d = v2.complex_value ();
 
   if (d == 0.0)
-    {
-      gripe_divide_by_zero ();
+    gripe_divide_by_zero ();
 
-      retval = octave_value (v1.complex_matrix_value () / d);
-    }
-  else
-    retval =  octave_value (v1.sparse_complex_matrix_value () / d);
+  retval =  octave_value (v1.sparse_complex_matrix_value () / d);
 
   return retval;
 }
@@ -129,8 +121,8 @@ DEFBINOP_FN (el_pow, sparse_complex_matrix, complex, elem_xpow)
 
 DEFBINOP (el_ldiv, sparse_complex_matrix, complex)
 {
-  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&, 
-		   const octave_complex&);
+  CAST_BINOP_ARGS (const octave_sparse_complex_matrix&,
+                   const octave_complex&);
 
   return octave_value
     (x_el_div (v2.complex_value (), v1.sparse_complex_matrix_value ()));
@@ -164,35 +156,29 @@ install_scm_cs_ops (void)
   INSTALL_BINOP (op_mul, octave_sparse_complex_matrix, octave_complex, mul);
   INSTALL_BINOP (op_div, octave_sparse_complex_matrix, octave_complex, div);
   INSTALL_BINOP (op_pow, octave_sparse_complex_matrix, octave_complex, pow);
-  INSTALL_BINOP (op_ldiv, octave_sparse_complex_matrix, octave_complex, 
-		 ldiv);
+  INSTALL_BINOP (op_ldiv, octave_sparse_complex_matrix, octave_complex,
+                 ldiv);
   INSTALL_BINOP (op_lt, octave_sparse_complex_matrix, octave_complex, lt);
   INSTALL_BINOP (op_le, octave_sparse_complex_matrix, octave_complex, le);
   INSTALL_BINOP (op_eq, octave_sparse_complex_matrix, octave_complex, eq);
   INSTALL_BINOP (op_ge, octave_sparse_complex_matrix, octave_complex, ge);
   INSTALL_BINOP (op_gt, octave_sparse_complex_matrix, octave_complex, gt);
   INSTALL_BINOP (op_ne, octave_sparse_complex_matrix, octave_complex, ne);
-  INSTALL_BINOP (op_el_mul, octave_sparse_complex_matrix, octave_complex, 
-		 el_mul);
-  INSTALL_BINOP (op_el_div, octave_sparse_complex_matrix, octave_complex, 
-		 el_div);
-  INSTALL_BINOP (op_el_pow, octave_sparse_complex_matrix, octave_complex, 
-		 el_pow);
-  INSTALL_BINOP (op_el_ldiv, octave_sparse_complex_matrix, octave_complex, 
-		 el_ldiv);
-  INSTALL_BINOP (op_el_and, octave_sparse_complex_matrix, octave_complex, 
-		 el_and);
-  INSTALL_BINOP (op_el_or, octave_sparse_complex_matrix, octave_complex, 
-		 el_or);
+  INSTALL_BINOP (op_el_mul, octave_sparse_complex_matrix, octave_complex,
+                 el_mul);
+  INSTALL_BINOP (op_el_div, octave_sparse_complex_matrix, octave_complex,
+                 el_div);
+  INSTALL_BINOP (op_el_pow, octave_sparse_complex_matrix, octave_complex,
+                 el_pow);
+  INSTALL_BINOP (op_el_ldiv, octave_sparse_complex_matrix, octave_complex,
+                 el_ldiv);
+  INSTALL_BINOP (op_el_and, octave_sparse_complex_matrix, octave_complex,
+                 el_and);
+  INSTALL_BINOP (op_el_or, octave_sparse_complex_matrix, octave_complex,
+                 el_or);
 
   INSTALL_CATOP (octave_sparse_complex_matrix, octave_complex, scm_cs);
 
   INSTALL_ASSIGNOP (op_asn_eq, octave_sparse_complex_matrix, octave_complex,
-		    assign);
+                    assign);
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
