@@ -1,7 +1,6 @@
 /*
 
-Copyright (C) 1996, 1997, 1999, 2000, 2002, 2003, 2004, 2005, 2006,
-              2007 John W. Eaton
+Copyright (C) 1996-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -30,26 +29,10 @@ along with Octave; see the file COPYING.  If not, see
 #include "Array.h"
 #include "Array.cc"
 
-#include "Array2.h"
+#include "ov.h"
 
-#include "ArrayN.h"
-#include "ArrayN.cc"
+#include "oct-sort.cc"
 
-#include "DiagArray2.h"
-#include "DiagArray2.cc"
+NO_INSTANTIATE_ARRAY_SORT (octave_value);
 
-#include "oct-obj.h"
-
-template class OCTINTERP_API Array<octave_value>;
-
-INSTANTIATE_ARRAY_ASSIGN (octave_value, octave_value, OCTINTERP_API);
-
-template class OCTINTERP_API Array2<octave_value>;
-
-template class OCTINTERP_API ArrayN<octave_value>;
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
+INSTANTIATE_ARRAY (octave_value, OCTINTERP_API);
