@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2001, 2006, 2007 Paul Kienzle
+Copyright (C) 2001-2011 Paul Kienzle
 
 This file is part of Octave.
 
@@ -71,16 +71,16 @@ typedef enum
     mxUINT32_CLASS,
     mxINT64_CLASS,
     mxUINT64_CLASS,
-    mxFUNCTION_CLASS,
+    mxFUNCTION_CLASS
   }
   mxClassID;
 
-typedef int mxLogical;
+typedef unsigned char mxLogical;
 
 /* typedef Uint16 mxChar; */
 typedef char mxChar;
 
-/* 
+/*
  * FIXME? Mathworks says these should be size_t on 64-bit system and when
  * mex is used with the -largearraydims flag, but why do that? Its better
  * to conform to the same indexing as the rest of Octave
@@ -90,6 +90,8 @@ typedef int mwIndex;
 
 #if ! defined (MXARRAY_TYPEDEFS_ONLY)
 
+#include <cstring>
+
 class octave_value;
 
 #define DO_MUTABLE_METHOD(RET_T, METHOD_CALL) \
@@ -132,7 +134,7 @@ public:
   mxArray (const octave_value& ov);
 
   mxArray (mxClassID id, mwSize ndims, const mwSize *dims,
-	   mxComplexity flag = mxREAL);
+           mxComplexity flag = mxREAL);
 
   mxArray (mxClassID id, const dim_vector& dv, mxComplexity flag = mxREAL);
 
@@ -147,7 +149,7 @@ public:
   mxArray (mwSize m, const char **str);
 
   mxArray (mxClassID id, mwSize m, mwSize n, mwSize nzmax,
-	   mxComplexity flag = mxREAL);
+           mxComplexity flag = mxREAL);
 
   mxArray (mwSize ndims, const mwSize *dims, int num_keys, const char **keys);
 
@@ -161,9 +163,9 @@ public:
 
   mxArray (mwSize m, mwSize n);
 
-  virtual mxArray *clone (void) const
+  virtual mxArray *dup (void) const
   {
-    mxArray *new_rep = rep->clone ();
+    mxArray *new_rep = rep->dup ();
 
     return new mxArray (new_rep, name);
   }
@@ -182,6 +184,8 @@ public:
 
   virtual int is_double (void) const { return rep->is_double (); }
 
+  virtual int is_function_handle (void) const { return rep->is_function_handle (); }
+
   virtual int is_int16 (void) const { return rep->is_int16 (); }
 
   virtual int is_int32 (void) const { return rep->is_int32 (); }
@@ -302,9 +306,9 @@ public:
 
     if (str)
       {
-	mwSize sz =  sizeof (mxChar) * (strlen (str) + 1);
-	retval = static_cast<char *> (mxArray::malloc (sz));
-	strcpy (retval, str);
+        mwSize sz =  sizeof (mxChar) * (strlen (str) + 1);
+        retval = static_cast<char *> (mxArray::malloc (sz));
+        strcpy (retval, str);
       }
 
     return retval;
