@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2002, 2003, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 2002-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -23,20 +23,14 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_octave_h)
 #define octave_octave_h 1
 
-#ifdef	__cplusplus
+#ifdef  __cplusplus
 extern "C" {
 #endif
 
 extern OCTINTERP_API int octave_main (int argc, char **argv, int embedded);
 
-#ifdef	__cplusplus
+#ifdef  __cplusplus
 }
 #endif
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
