@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2000, 2005, 2006, 2007 John W. Eaton
+Copyright (C) 2000-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -45,9 +45,3 @@ extern char *strsignal (int);
 #endif
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
