@@ -1,12 +1,14 @@
 /*
 
-Copyright (C) 2004 David Bateman
+Copyright (C) 2004-2011 David Bateman
 Copyright (C) 1998-2004 Andy Adler
 
+This file is part of Octave.
+
 Octave is free software; you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
-Free Software Foundation; either version 2, or (at your option) any
-later version.
+Free Software Foundation; either version 3 of the License, or (at your
+option) any later version.
 
 Octave is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
@@ -14,9 +16,8 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-Boston, MA 02110-1301, USA.
+along with Octave; see the file COPYING.  If not, see
+<http://www.gnu.org/licenses/>.
 
 */
 
@@ -45,8 +46,8 @@ static inline int
 xisint (double x)
 {
   return (D_NINT (x) == x
-	  && ((x >= 0 && x < INT_MAX)
-	      || (x <= 0 && x > INT_MIN)));
+          && ((x >= 0 && x < INT_MAX)
+              || (x <= 0 && x > INT_MIN)));
 }
 
 
@@ -66,61 +67,61 @@ xpow (const SparseMatrix& a, double b)
   else
     {
       if (static_cast<int> (b) == b)
-	{
-	  int btmp = static_cast<int> (b);
-	  if (btmp == 0)
-	    {
-	      SparseMatrix tmp = SparseMatrix (nr, nr, nr);
-	      for (octave_idx_type i = 0; i < nr; i++)
-		{
-		  tmp.data (i) = 1.0;
-		  tmp.ridx (i) = i;
-		}
-	      for (octave_idx_type i = 0; i < nr + 1; i++)
-		tmp.cidx (i) = i;
+        {
+          int btmp = static_cast<int> (b);
+          if (btmp == 0)
+            {
+              SparseMatrix tmp = SparseMatrix (nr, nr, nr);
+              for (octave_idx_type i = 0; i < nr; i++)
+                {
+                  tmp.data (i) = 1.0;
+                  tmp.ridx (i) = i;
+                }
+              for (octave_idx_type i = 0; i < nr + 1; i++)
+                tmp.cidx (i) = i;
 
-	      retval = tmp;
-	    }
-	  else
-	    {
-	      SparseMatrix atmp;
-	      if (btmp < 0)
-		{
-		  btmp = -btmp;
+              retval = tmp;
+            }
+          else
+            {
+              SparseMatrix atmp;
+              if (btmp < 0)
+                {
+                  btmp = -btmp;
 
-		  octave_idx_type info;
-		  double rcond = 0.0;
-		  MatrixType mattyp (a);
+                  octave_idx_type info;
+                  double rcond = 0.0;
+                  MatrixType mattyp (a);
 
-		  atmp = a.inverse (mattyp, info, rcond, 1);
+                  atmp = a.inverse (mattyp, info, rcond, 1);
 
-		  if (info == -1)
-		    warning ("inverse: matrix singular to machine\
+                  if (info == -1)
+                    warning ("inverse: matrix singular to machine\
  precision, rcond = %g", rcond);
-		}
-	      else
-		atmp = a;
+                }
+              else
+                atmp = a;
 
-	      SparseMatrix result (atmp);
+              SparseMatrix result (atmp);
 
-	      btmp--;
+              btmp--;
 
-	      while (btmp > 0)
-		{
-		  if (btmp & 1)
-		    result = result * atmp;
+              while (btmp > 0)
+                {
+                  if (btmp & 1)
+                    result = result * atmp;
 
-		  btmp >>= 1;
+                  btmp >>= 1;
 
-		  if (btmp > 0)
-		    atmp = atmp * atmp;
-		}
+                  if (btmp > 0)
+                    atmp = atmp * atmp;
+                }
 
-	      retval = result;
-	    }
-	}
+              retval = result;
+            }
+        }
       else
-	error ("use full(a) ^ full(b)");
+        error ("use full(a) ^ full(b)");
     }
 
   return retval;
@@ -139,61 +140,61 @@ xpow (const SparseComplexMatrix& a, double b)
   else
     {
       if (static_cast<int> (b) == b)
-	{
-	  int btmp = static_cast<int> (b);
-	  if (btmp == 0)
-	    {
-	      SparseMatrix tmp = SparseMatrix (nr, nr, nr);
-	      for (octave_idx_type i = 0; i < nr; i++)
-		{
-		  tmp.data (i) = 1.0;
-		  tmp.ridx (i) = i;
-		}
-	      for (octave_idx_type i = 0; i < nr + 1; i++)
-		tmp.cidx (i) = i;
+        {
+          int btmp = static_cast<int> (b);
+          if (btmp == 0)
+            {
+              SparseMatrix tmp = SparseMatrix (nr, nr, nr);
+              for (octave_idx_type i = 0; i < nr; i++)
+                {
+                  tmp.data (i) = 1.0;
+                  tmp.ridx (i) = i;
+                }
+              for (octave_idx_type i = 0; i < nr + 1; i++)
+                tmp.cidx (i) = i;
 
-	      retval = tmp;
-	    }
-	  else
-	    {
-	      SparseComplexMatrix atmp;
-	      if (btmp < 0)
-		{
-		  btmp = -btmp;
+              retval = tmp;
+            }
+          else
+            {
+              SparseComplexMatrix atmp;
+              if (btmp < 0)
+                {
+                  btmp = -btmp;
 
-		  octave_idx_type info;
-		  double rcond = 0.0;
-		  MatrixType mattyp (a);
+                  octave_idx_type info;
+                  double rcond = 0.0;
+                  MatrixType mattyp (a);
 
-		  atmp = a.inverse (mattyp, info, rcond, 1);
+                  atmp = a.inverse (mattyp, info, rcond, 1);
 
-		  if (info == -1)
-		    warning ("inverse: matrix singular to machine\
+                  if (info == -1)
+                    warning ("inverse: matrix singular to machine\
  precision, rcond = %g", rcond);
-		}
-	      else
-		atmp = a;
+                }
+              else
+                atmp = a;
 
-	      SparseComplexMatrix result (atmp);
+              SparseComplexMatrix result (atmp);
 
-	      btmp--;
+              btmp--;
 
-	      while (btmp > 0)
-		{
-		  if (btmp & 1)
-		    result = result * atmp;
+              while (btmp > 0)
+                {
+                  if (btmp & 1)
+                    result = result * atmp;
 
-		  btmp >>= 1;
+                  btmp >>= 1;
 
-		  if (btmp > 0)
-		    atmp = atmp * atmp;
-		}
+                  if (btmp > 0)
+                    atmp = atmp * atmp;
+                }
 
-	      retval = result;
-	    }
-	}
+              retval = result;
+            }
+        }
       else
-	error ("use full(a) ^ full(b)");
+        error ("use full(a) ^ full(b)");
     }
 
   return retval;
@@ -245,13 +246,13 @@ elem_xpow (double a, const SparseMatrix& b)
       ComplexMatrix result (nr, nc);
 
       for (octave_idx_type j = 0; j < nc; j++)
-	{
-	  for (octave_idx_type i = 0; i < nr; i++)
-	    {
-	      OCTAVE_QUIT;
-	      result (i, j) = std::pow (atmp, b(i,j));
-	    }
-	}
+        {
+          for (octave_idx_type i = 0; i < nr; i++)
+            {
+              octave_quit ();
+              result (i, j) = std::pow (atmp, b(i,j));
+            }
+        }
 
       retval = result;
     }
@@ -260,13 +261,13 @@ elem_xpow (double a, const SparseMatrix& b)
       Matrix result (nr, nc);
 
       for (octave_idx_type j = 0; j < nc; j++)
-	{
-	  for (octave_idx_type i = 0; i < nr; i++)
-	    {
-	      OCTAVE_QUIT;
-	      result (i, j) = std::pow (a, b(i,j));
-	    }
-	}
+        {
+          for (octave_idx_type i = 0; i < nr; i++)
+            {
+              octave_quit ();
+              result (i, j) = std::pow (a, b(i,j));
+            }
+        }
 
       retval = result;
     }
@@ -287,10 +288,10 @@ elem_xpow (double a, const SparseComplexMatrix& b)
   for (octave_idx_type j = 0; j < nc; j++)
     {
       for (octave_idx_type i = 0; i < nr; i++)
-	{
-	  OCTAVE_QUIT;
-	  result (i, j) = std::pow (atmp, b(i,j));
-	}
+        {
+          octave_quit ();
+          result (i, j) = std::pow (atmp, b(i,j));
+        }
     }
 
   return result;
@@ -300,13 +301,13 @@ elem_xpow (double a, const SparseComplexMatrix& b)
 octave_value
 elem_xpow (const SparseMatrix& a, double b)
 {
-  // FIXME What should a .^ 0 give?? Matlab gives a 
+  // FIXME What should a .^ 0 give?? Matlab gives a
   // sparse matrix with same structure as a, which is strictly
   // incorrect. Keep compatiability.
 
   octave_value retval;
 
-  octave_idx_type nz = a.nzmax ();
+  octave_idx_type nz = a.nnz ();
 
   if (b <= 0.0)
     {
@@ -314,55 +315,55 @@ elem_xpow (const SparseMatrix& a, double b)
       octave_idx_type nc = a.cols ();
 
       if (static_cast<int> (b) != b && a.any_element_is_negative ())
-	{
-	  ComplexMatrix result (nr, nc, Complex (std::pow (0.0, b)));
+        {
+          ComplexMatrix result (nr, nc, Complex (std::pow (0.0, b)));
 
-	  // FIXME -- avoid apparent GNU libm bug by
-	  // converting A and B to complex instead of just A.
-	  Complex btmp (b);
+          // FIXME -- avoid apparent GNU libm bug by
+          // converting A and B to complex instead of just A.
+          Complex btmp (b);
 
-	  for (octave_idx_type j = 0; j < nc; j++)
-	    for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	      {
-		OCTAVE_QUIT;
-	      
-		Complex atmp (a.data (i));
-		
-		result (a.ridx(i), j) = std::pow (atmp, btmp);
-	      }
+          for (octave_idx_type j = 0; j < nc; j++)
+            for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
+              {
+                octave_quit ();
 
-	  retval = octave_value (result);
-	}
+                Complex atmp (a.data (i));
+
+                result (a.ridx(i), j) = std::pow (atmp, btmp);
+              }
+
+          retval = octave_value (result);
+        }
       else
-	{
-	  Matrix result (nr, nc, (std::pow (0.0, b)));
+        {
+          Matrix result (nr, nc, (std::pow (0.0, b)));
 
-	  for (octave_idx_type j = 0; j < nc; j++)
-	    for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	      {
-		OCTAVE_QUIT;
-		result (a.ridx(i), j) = std::pow (a.data (i), b);
-	      }
+          for (octave_idx_type j = 0; j < nc; j++)
+            for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
+              {
+                octave_quit ();
+                result (a.ridx(i), j) = std::pow (a.data (i), b);
+              }
 
-	  retval = octave_value (result);
-	}
+          retval = octave_value (result);
+        }
     }
   else if (static_cast<int> (b) != b && a.any_element_is_negative ())
     {
       SparseComplexMatrix result (a);
 
       for (octave_idx_type i = 0; i < nz; i++)
-	{
-	  OCTAVE_QUIT;
+        {
+          octave_quit ();
 
-	  // FIXME -- avoid apparent GNU libm bug by
-	  // converting A and B to complex instead of just A.
+          // FIXME -- avoid apparent GNU libm bug by
+          // converting A and B to complex instead of just A.
 
-	  Complex atmp (a.data (i));
-	  Complex btmp (b);
+          Complex atmp (a.data (i));
+          Complex btmp (b);
 
-	  result.data (i) = std::pow (atmp, btmp);
-	}
+          result.data (i) = std::pow (atmp, btmp);
+        }
 
       result.maybe_compress (true);
 
@@ -373,10 +374,10 @@ elem_xpow (const SparseMatrix& a, double b)
       SparseMatrix result (a);
 
       for (octave_idx_type i = 0; i < nz; i++)
-	{
-	  OCTAVE_QUIT;
-	  result.data (i) = std::pow (a.data (i), b);
-	}
+        {
+          octave_quit ();
+          result.data (i) = std::pow (a.data (i), b);
+        }
 
       result.maybe_compress (true);
 
@@ -408,15 +409,15 @@ elem_xpow (const SparseMatrix& a, const SparseMatrix& b)
   for (octave_idx_type j = 0; j < nc; j++)
     for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
       {
-	if (a.data(i) < 0.0)
-	  {
-	    double btmp = b (a.ridx(i), j);
-	    if (static_cast<int> (btmp) != btmp)
-	      {
-		convert_to_complex = 1;
-		goto done;
-	      }
-	  }
+        if (a.data(i) < 0.0)
+          {
+            double btmp = b (a.ridx(i), j);
+            if (static_cast<int> (btmp) != btmp)
+              {
+                convert_to_complex = 1;
+                goto done;
+              }
+          }
       }
 
 done:
@@ -431,14 +432,14 @@ done:
       SparseComplexMatrix complex_result (nr, nc, Complex(1.0, 0.0));
 
       for (octave_idx_type j = 0; j < nc; j++)
-	{
-	  for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	    {
-	      OCTAVE_QUIT;
-	      complex_result.xelem(a.ridx(i), j) =
-		std::pow (Complex(a.data(i)), Complex(b(a.ridx(i), j)));
-	    }
-	}
+        {
+          for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
+            {
+              octave_quit ();
+              complex_result.xelem(a.ridx(i), j) =
+                std::pow (Complex(a.data(i)), Complex(b(a.ridx(i), j)));
+            }
+        }
       complex_result.maybe_compress (true);
       retval = complex_result;
     }
@@ -447,14 +448,14 @@ done:
       SparseMatrix result (nr, nc, 1.0);
 
       for (octave_idx_type j = 0; j < nc; j++)
-	{
-	  for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	    {
-	      OCTAVE_QUIT;
-	      result.xelem(a.ridx(i), j) = std::pow (a.data(i), 
-						     b (a.ridx(i), j));
-	    }
-	}
+        {
+          for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
+            {
+              octave_quit ();
+              result.xelem(a.ridx(i), j) = std::pow (a.data(i),
+                                                     b (a.ridx(i), j));
+            }
+        }
       result.maybe_compress (true);
       retval = result;
     }
@@ -473,15 +474,15 @@ elem_xpow (const SparseMatrix& a, const Complex& b)
     retval = octave_value (NDArray (a.dims (), 1));
   else
     {
-      octave_idx_type nz = a.nzmax ();
+      octave_idx_type nz = a.nnz ();
       SparseComplexMatrix result (a);
-      
+
       for (octave_idx_type i = 0; i < nz; i++)
-	{
-	  OCTAVE_QUIT;
-	  result.data (i) = std::pow (Complex (a.data (i)), b);
-	}
-  
+        {
+          octave_quit ();
+          result.data (i) = std::pow (Complex (a.data (i)), b);
+        }
+
       result.maybe_compress (true);
 
       retval = result;
@@ -510,10 +511,10 @@ elem_xpow (const SparseMatrix& a, const SparseComplexMatrix& b)
   for (octave_idx_type j = 0; j < nc; j++)
     {
       for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	{
-	  OCTAVE_QUIT;
-	  result.xelem(a.ridx(i), j) = std::pow (a.data(i), b (a.ridx(i), j));
-	}
+        {
+          octave_quit ();
+          result.xelem(a.ridx(i), j) = std::pow (a.data(i), b (a.ridx(i), j));
+        }
     }
 
   result.maybe_compress (true);
@@ -533,14 +534,14 @@ elem_xpow (const Complex& a, const SparseMatrix& b)
   for (octave_idx_type j = 0; j < nc; j++)
     {
       for (octave_idx_type i = 0; i < nr; i++)
-	{
-	  OCTAVE_QUIT;
-	  double btmp = b (i, j);
-	  if (xisint (btmp))
-	    result (i, j) = std::pow (a, static_cast<int> (btmp));
-	  else
-	    result (i, j) = std::pow (a, btmp);
-	}
+        {
+          octave_quit ();
+          double btmp = b (i, j);
+          if (xisint (btmp))
+            result (i, j) = std::pow (a, static_cast<int> (btmp));
+          else
+            result (i, j) = std::pow (a, btmp);
+        }
     }
 
   return result;
@@ -557,8 +558,8 @@ elem_xpow (const Complex& a, const SparseComplexMatrix& b)
   for (octave_idx_type j = 0; j < nc; j++)
     for (octave_idx_type i = 0; i < nr; i++)
       {
-	OCTAVE_QUIT;
-	result (i, j) = std::pow (a, b (i, j));
+        octave_quit ();
+        result (i, j) = std::pow (a, b (i, j));
       }
 
   return result;
@@ -578,49 +579,49 @@ elem_xpow (const SparseComplexMatrix& a, double b)
       ComplexMatrix result (nr, nc, Complex (std::pow (0.0, b)));
 
       if (xisint (b))
-	{
-	  for (octave_idx_type j = 0; j < nc; j++)
-	    for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	      {
-		OCTAVE_QUIT;
-		result (a.ridx(i), j) = 
-		  std::pow (a.data (i), static_cast<int> (b));
-	      }
-	}
+        {
+          for (octave_idx_type j = 0; j < nc; j++)
+            for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
+              {
+                octave_quit ();
+                result (a.ridx(i), j) =
+                  std::pow (a.data (i), static_cast<int> (b));
+              }
+        }
       else
-	{
-	  for (octave_idx_type j = 0; j < nc; j++)
-	    for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	      {
-		OCTAVE_QUIT;
-		result (a.ridx(i), j) = std::pow (a.data (i), b);
-	      }
-	}  
+        {
+          for (octave_idx_type j = 0; j < nc; j++)
+            for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
+              {
+                octave_quit ();
+                result (a.ridx(i), j) = std::pow (a.data (i), b);
+              }
+        }
 
       retval = result;
     }
   else
     {
-      octave_idx_type nz = a.nzmax ();
+      octave_idx_type nz = a.nnz ();
 
       SparseComplexMatrix result (a);
-  
+
       if (xisint (b))
-	{
-	  for (octave_idx_type i = 0; i < nz; i++)
-	    {
-	      OCTAVE_QUIT;
-	      result.data (i) = std::pow (a.data (i), static_cast<int> (b));
-	    }
-	}
+        {
+          for (octave_idx_type i = 0; i < nz; i++)
+            {
+              octave_quit ();
+              result.data (i) = std::pow (a.data (i), static_cast<int> (b));
+            }
+        }
       else
-	{
-	  for (octave_idx_type i = 0; i < nz; i++)
-	    {
-	      OCTAVE_QUIT;
-	      result.data (i) = std::pow (a.data (i), b);
-	    }
-	}  
+        {
+          for (octave_idx_type i = 0; i < nz; i++)
+            {
+              octave_quit ();
+              result.data (i) = std::pow (a.data (i), b);
+            }
+        }
 
       result.maybe_compress (true);
 
@@ -650,17 +651,17 @@ elem_xpow (const SparseComplexMatrix& a, const SparseMatrix& b)
   for (octave_idx_type j = 0; j < nc; j++)
     {
       for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	{
-	  OCTAVE_QUIT;
-	  double btmp = b (a.ridx(i), j);
-	  Complex tmp;
+        {
+          octave_quit ();
+          double btmp = b (a.ridx(i), j);
+          Complex tmp;
 
-	  if (xisint (btmp))
-	    result.xelem(a.ridx(i), j) = std::pow (a.data (i), 
-					      static_cast<int> (btmp));
-	  else
-	    result.xelem(a.ridx(i), j) = std::pow (a.data (i), btmp);
-	}
+          if (xisint (btmp))
+            result.xelem(a.ridx(i), j) = std::pow (a.data (i),
+                                              static_cast<int> (btmp));
+          else
+            result.xelem(a.ridx(i), j) = std::pow (a.data (i), btmp);
+        }
     }
 
   result.maybe_compress (true);
@@ -680,18 +681,18 @@ elem_xpow (const SparseComplexMatrix& a, const Complex& b)
   else
     {
 
-      octave_idx_type nz = a.nzmax ();
+      octave_idx_type nz = a.nnz ();
 
       SparseComplexMatrix result (a);
 
       for (octave_idx_type i = 0; i < nz; i++)
-	{
-	  OCTAVE_QUIT;
-	  result.data (i) = std::pow (a.data (i), b);
-	}
+        {
+          octave_quit ();
+          result.data (i) = std::pow (a.data (i), b);
+        }
 
       result.maybe_compress (true);
-      
+
       retval = result;
     }
 
@@ -715,21 +716,15 @@ elem_xpow (const SparseComplexMatrix& a, const SparseComplexMatrix& b)
     }
 
   SparseComplexMatrix result (nr, nc, Complex(1.0, 0.0));
-  for (octave_idx_type j = 0; j < nc; j++) 
+  for (octave_idx_type j = 0; j < nc; j++)
     {
       for (octave_idx_type i = a.cidx(j); i < a.cidx(j+1); i++)
-	{
-	  OCTAVE_QUIT;
-	  result.xelem(a.ridx(i), j) = std::pow (a.data (i), b (a.ridx(i), j));
-	}
+        {
+          octave_quit ();
+          result.xelem(a.ridx(i), j) = std::pow (a.data (i), b (a.ridx(i), j));
+        }
     }
   result.maybe_compress (true);
 
   return result;
 }
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
