@@ -1,8 +1,6 @@
 /*
 
-Copyright (C) 1992, 1993, 1994, 1996, 1997, 1998, 1999, 2000, 2001,
-              2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
-              John W. Eaton
+Copyright (C) 1992-2011 John W. Eaton
 
 This file is part of Octave.
 
@@ -25,13 +23,15 @@ along with Octave; see the file COPYING.  If not, see
 #if !defined (octave_version_h)
 #define octave_version_h 1
 
-#define OCTAVE_VERSION "3.2.4"
+#define OCTAVE_VERSION "3.4.0"
 
-#define OCTAVE_API_VERSION "api-v37"
+#define OCTAVE_API_VERSION_NUMBER 47
 
-#define OCTAVE_RELEASE_DATE "2010-01-22"
+#define OCTAVE_API_VERSION "api-v47+"
 
-#define OCTAVE_COPYRIGHT "Copyright (C) 2009 John W. Eaton and others."
+#define OCTAVE_RELEASE_DATE "2011-02-08"
+
+#define OCTAVE_COPYRIGHT "Copyright (C) 2011 John W. Eaton and others."
 
 // This is the first line printed by --version.  The GNU coding
 // standards say that the version number should follow the last space
@@ -60,8 +60,7 @@ FITNESS FOR A PARTICULAR PURPOSE." ARG
 For more information, visit http://www.octave.org/help-wanted.html"
 
 #define OCTAVE_BUGS_STATEMENT \
-  "Report bugs to <bug@octave.org> (but first, please read\n\
-http://www.octave.org/bugs.html to learn how to write a helpful report)."
+  "Read http://www.octave.org/bugs.html to learn how to submit bug reports."
 
 #define OCTAVE_NAME_VERSION_AND_COPYRIGHT \
   OCTAVE_NAME_AND_VERSION "\n" \
@@ -91,9 +90,3 @@ http://www.octave.org/bugs.html to learn how to write a helpful report)."
   "For information about changes from previous versions, type `news'."
 
 #endif
-
-/*
-;;; Local Variables: ***
-;;; mode: C++ ***
-;;; End: ***
-*/
